
# Oject Storage Extension Samples

## Overview
[VMware Cloud Director Object Storage Extension](https://docs.vmware.com/en/VMware-Cloud-Director-Object-Storage-Extension/index.html) (OSE) is a midware which provides the capability of consuming object storage services for Cloud Director users.

Since version 2.0, OSE opens the extensibility framework for vendor and community to integrate third-party S3 compliant object storage platforms with Cloud Director. The extensibility framework offers a set of REST APIs for the vendor to implement upon the third-party object storage platform, called Object Storage Interoperability Services (OSIS). Below diagram gives the overview of OSE architecture and OSIS' position in the system.

![OSE Architecture](assets/ose-architecture-2-0.png?raw=true)

This open source project includes sample implentation of extensively integrated object storage platforms for OSE, but not limited to these.

## Samples
[OSIS Instruction](vmware-ose-osis-instruction/) gives overview of OSIS deveployment and deployment.
* [OSIS CEPH Reference Implementation](vmware-ose-osis-ceph-ri/) is an implementation of OSIS for CEPH.
* [OSIS Stub](vmware-ose-osis-stub/) is a Java REST client stub of OSIS specification.
* [OSIS Verifier](vmware-ose-osis-verifier/) is a tool to verify the readiness and compatibiity of the OSIS implementation.

[Common Test Suites](vmware-ose-common-test-suites/) is developed based on [boto3](https://github.com/boto/boto3) to verify the compatibility of OSE certified storage platforms, like Clouian HyperStore and DELL ECS.

## Preliminaries

- The test environment of vCD, OSE and storage platform needs to be setup according to OSE release notes.

- Have _python 3.0+_ installed

- Have _virtualenv_ installed
    
    `pip install virtualenv`

- Have required libraries installed

    `pip install -r requirements.txt`
- Brew Install Allure

    `brew install allure`
- NPM Install Allure

    `npm install -g allure-commandline`


### Command Options
**Required parameters:** 

- Profile: Specify the test profile used
  
    `--env_profile` # default env_cloudian.yml
  
- Project: Specify the project
  
    `--project_name` # default osis_auto

- More options

    `python ose_test_runner.py -h`

**Update user profile**

### Edit User Profile

env_{ceph;cloudian} maintains temporary information the cases use.

## Run

You can run with below command using different options.

- Run with options

  - Run test sequentially
    `python ose_test_runner.py --env_file=env_osis_ceph.yml --project_name=osis_auto`
     
  - Run all test parallel
      `python ose_test_runner.py --env_file=env_osis_ceph.yml --project_name=osis_auto --processes=20 --logging_level=INFO`
     
  - Run test with specified tag
      `python ose_test_runner.py --env_file=env_osis_ceph.yml --project_name=osis_auto --tags=bucket_acl --logging_level=INFO`


## Contributing

The object-storage-extension-samples project team welcomes contributions from the community. If you wish to contribute code and you have not signed our contributor license agreement (CLA), our bot will update the issue when you open a Pull Request. For any questions about the CLA process, please refer to our [FAQ](https://cla.vmware.com/faq).

All contributions to this repository must be signed as described on that page. Your signature certifies that you wrote the patch or have the right to pass it on as an open-source patch. For more detailed information, refer to [CONTRIBUTING.md](CONTRIBUTING.md).
