import os
from configparser import ConfigParser

import numpy as np
import pandas as pd
import yaml
from yaml import YAMLError
from yaml.loader import BaseLoader, SafeLoader, FullLoader


class FileUtils:

    def __init__(self):
        pass

    @staticmethod
    def load_yaml(yaml_file_path):
        # Program to convert yaml file to dictionary
        # opening a file
        with open(yaml_file_path, 'r') as stream:
            try:
                # Converts yaml document to python object
                yaml_dict = yaml.safe_load(stream)
                # Printing dictionary
                return yaml_dict
            except YAMLError as e:
                raise YAMLError("Fail to load yaml file: " + str(e))

    @staticmethod
    def load_yaml_with_base_loader(yaml_file_path):
        # opening a file
        with open(yaml_file_path, 'r') as stream:

            try:
                # Converts yaml document to python object
                yaml_dict = yaml.load(stream, Loader=BaseLoader)
                return yaml_dict
            except YAMLError as e:
                raise YAMLError("Fail to load yaml file: " + str(e))

    @staticmethod
    def load_yaml_with_safe_loader(yaml_file_path):
        # opening a file
        with open(yaml_file_path, 'r') as stream:
            try:
                # Converts yaml document to python object
                yaml_dict = yaml.load(stream, Loader=SafeLoader)
                return yaml_dict
            except YAMLError as e:
                raise YAMLError("Fail to load yaml file: " + str(e))

    @staticmethod
    def load_yaml_with_full_loader(yaml_file_path):
        # opening a file
        with open(yaml_file_path, 'r') as stream:
            try:
                return_dict = {}
                # Converts yaml document to python object
                yaml_dict = yaml.load(stream, Loader=FullLoader)
                for key, val in yaml_dict.items():
                    if isinstance(val, dict):
                        for key1, val1 in val.items():
                            if isinstance(val1, dict):
                                for key2, val2 in val1.items():
                                    os.environ[key + "_" + key1 + "_" + key2] = str(val2)
                                    return_dict[key + "_" + key1 + "_" + key2] = str(val2)
                            else:
                                os.environ[key + "_" + key1] = str(val1)
                                return_dict[key + "_" + key1] = str(val1)
                    else:
                        os.environ[key] = str(val)
                        return_dict[key] = str(val)

                return return_dict
            except YAMLError as e:
                raise YAMLError("Fail to load yaml file: " + str(e))

    @staticmethod
    def update_data_in_yaml_file(yaml_file_path, tenant_name="", tenant_user="", parameter_name="", parameter_value=""):
        try:
            with open(yaml_file_path, 'r') as f:
                yaml_dict = yaml.load(f, Loader=yaml.FullLoader) or {}
                # update tenant related information
                if tenant_user:
                    yaml_dict["vcd-site"][tenant_name][tenant_user][parameter_name] = parameter_value
                else:
                    if tenant_name:
                        yaml_dict["vcd-site"][tenant_name][parameter_name] = parameter_value
                    else:
                        yaml_dict["vcd-site"][parameter_name] = parameter_value
            # yaml_dict.update(kwargs)
            with open(yaml_file_path, 'w') as f:
                yaml.dump(yaml_dict, f)
        except YAMLError as e:
            raise YAMLError("Fail to update yaml file: " + str(e))

    @staticmethod
    def is_nan(nan):
        return nan != nan

    @staticmethod
    def load_csv(csv_file_path):
        data_list = []
        df = pd.read_csv(csv_file_path)
        # Replace all Nan values to empty string
        df = df.replace(np.nan, '', regex=True)
        # Get columns list
        list_of_columns = df.columns.tolist()
        # Use list comprehension to create a list of lists from Dataframe rows
        list_of_rows = [list(row) for row in df.values]
        for list_of_row in list_of_rows:
            data_dict = {}
            for index in range(len(list_of_columns)):
                if list_of_row[index]:
                    data_dict[list_of_columns[index]] = list_of_row[index]
            data_list.append(data_dict)
        return data_list

    @staticmethod
    def load_xlsx(excel_file_path=None, sheet_name=None):
        try:
            sheet_data = {}
            excel_sheet_names = (pd.ExcelFile(excel_file_path)).sheet_names
            if sheet_name:
                df = pd.read_excel(excel_file_path, sheet_name=sheet_name)
                # Replace all Nan values to empty string
                df = df.replace(np.nan, '', regex=True)
                # Get columns list
                list_of_columns = df.columns.tolist()
                # Use list comprehension to create a list of lists from Dataframe rows
                list_of_rows = [list(row) for row in df.values]
                for list_of_row in list_of_rows:
                    data_dict = {}
                    data_list = []
                    for index in range(len(list_of_columns)):
                        if list_of_row[index]:
                            data_dict[list_of_columns[index]] = list_of_row[index]
                    data_list.append(data_dict)
                sheet_data[sheet_name] = data_list
            else:
                for sheet_name in excel_sheet_names:
                    df = pd.read_excel(excel_file_path, sheet_name=sheet_name)
                    # Replace all Nan values to empty string
                    df = df.replace(np.nan, '', regex=True)
                    # Get columns list
                    list_of_columns = df.columns.tolist()
                    # Use list comprehension to create a list of lists from Dataframe rows
                    list_of_rows = [list(row) for row in df.values]
                    for list_of_row in list_of_rows:
                        data_dict = {}
                        data_list = []
                        for index in range(len(list_of_columns)):
                            if list_of_row[index]:
                                data_dict[list_of_columns[index]] = list_of_row[index]
                        data_list.append(data_dict)
                    sheet_data[sheet_name] = data_list
            return data_list
        except Exception as ex:
            raise Exception("Failed to load xlsx: " + str(ex))

    @staticmethod
    def load_ini_file(ini_file_path):
        cfg = ConfigParser()
        if os.path.exists(ini_file_path):
            cfg.read(ini_file_path)
        else:
            raise Exception("config ini file '" + ini_file_path + "' is not found as expected")
        return cfg

    @staticmethod
    def update_ini_file(ini_file_path, section_name, field_name, field_value):
        try:
            config_object = ConfigParser()
            config_object.read(ini_file_path)
            if section_name:
                section = config_object[section_name]
            else:
                section = config_object
            # update platform
            section[field_name] = field_value

            with open(ini_file_path, 'w') as config_file:
                config_object.write(config_file)
        except Exception as ex:
            raise Exception("Failed to update ini file: " + str(ex))

    @staticmethod
    def read_data_from_ini_file(cfg, section_name, field_name):
        return cfg[section_name][field_name]
