# -*- coding:utf-8 -*-

###########################################
# Function: Logging for testing
# Input: None
# Output: Logging to stdout and log file,
#        log file will be named by execute time, log path will be execute current path
###########################################

import logging
import os
import time
import threading

import allure
import colorlog


class TestLog:
    lock = threading.Lock()
    LOG_TYPE_TEST_EXECUTION = "Test Execution"
    LOG_TYPE_TEST_VALIDATION = "Test Validation"
    LOG_TYPE_TEST_EXCEPTION = "Test Exception"
    LOG_TYPE_TEST_DATA_CLEANUP = "Test DataCleanUp"
    LOG_TYPE_TEST_DATA_PREPARATION = "Test DataPreparation"
    _default_log_name = "OSE-Auto-Test"
    _default_log_level = "INFO"

    def __init__(self):
        self.logger = logging.getLogger()
        self.name = "OSE-Auto-Test"

    def set_logger(self, log_path=None, log_name=_default_log_name, logging_level=_default_log_level):
        """

        :param log_path:
        :param log_name:
        :param logging_level:
        :return:
        """
        my_logger_level = ""
        # create logger
        TestLog.lock.acquire()

        log_time = time.strftime("%Y-%m-%d-%H-%M-%S")
        # define handler output formats
        format_str = '%(asctime)s.%(msecs)d %(levelname)-8s [%(filename)s:%(lineno)d] %(message)s'
        date_format = '%Y-%m-%d %H:%M:%S'
        color_format = '%(log_color)s' + format_str
        colors = {'DEBUG': 'green',
                  'INFO': 'cyan',
                  'WARNING': 'bold_yellow',
                  'ERROR': 'bold_red',
                  'CRITICAL': 'bold_purple'}
        formatter = colorlog.ColoredFormatter(color_format, date_format,
                                              log_colors=colors)
        if log_path and os.path.isfile(log_path):
            my_log_file = log_path
        else:
            my_log_path = os.path.join(os.getcwd(),"ose_auto","test_log", time.strftime("%Y-%m-%d"))
            if not os.path.exists(my_log_path):
                os.makedirs(my_log_path)
            my_log_file = my_log_path + "/" + log_name + '@test_log_' + log_time + '.log'
        os.environ["test_log_file"] = my_log_file
        if not len(self.logger.handlers):
            if "logging_level" in os.environ:
                my_logger_level = os.environ["logging_level"]
            if my_logger_level:
                logging.basicConfig(filename=my_log_file, level=my_logger_level, format=format_str)
            else:
                # use the default logging level
                logging.basicConfig(filename=my_log_file, level=logging_level, format=format_str)

            # logging handler
            log_file_handler = logging.FileHandler(my_log_file, 'a', encoding='utf-8')  # append python3
            log_file_handler.setLevel(logging.INFO)
            log_file_handler.setFormatter(formatter)

            # create console handler
            stdout_handler = logging.StreamHandler()
            stdout_handler.setLevel(logging.DEBUG)
            stdout_handler.setFormatter(formatter)

            # add the handler to logger
            self.logger.addHandler(log_file_handler)
            self.logger.addHandler(stdout_handler)

            # close the log file
            log_file_handler.close()
            stdout_handler.close()
        elif len(self.logger.handlers) <= 1:
            for h in self.logger.handlers:
                self.logger.removeHandler(h)
            if "logging_level" in os.environ:
                my_logger_level = os.environ["logging_level"]
            if my_logger_level:
                logging.basicConfig(filename=my_log_file, level=my_logger_level, format=format_str)
            else:
                logging.basicConfig(filename=my_log_file, level=logging_level, format=format_str)

            # logging handler
            log_file_handler = logging.FileHandler(my_log_file, 'a', encoding='utf-8')  # append python3
            log_file_handler.setLevel(logging.INFO)
            log_file_handler.setFormatter(formatter)

            # create console handler
            stdout_handler = logging.StreamHandler()
            stdout_handler.setLevel(logging.DEBUG)
            stdout_handler.setFormatter(formatter)

            # add the handler to logger
            self.logger.addHandler(log_file_handler)
            self.logger.addHandler(stdout_handler)

            # close the log file
            log_file_handler.close()
            stdout_handler.close()
        TestLog.lock.release()
        return self.logger

    def get_logger(self):
        return self.logger

    @staticmethod
    def attach_allure_log(log_message, log_type):
        """
        Attach log to allure test report
        :param log_message:
        :param log_type: the valid values should be: Test Execution, Test Validation, Test Exception,Test DataCleanUp,Test DataTearDown, Test DataPreparation
        :return:
        """
        allure.attach(log_message, log_type, allure.attachment_type.TEXT)
