# -- FILE: features/environment.py
import os

from behave.model import Scenario
from behave.model_core import Status

from common_libraries.boto3_client import Boto3Client
from common_libraries.data_generator import DataGenerator
from common_libraries.data_providers.file_utils import FileUtils
from common_libraries.logger import TestLog

BEHAVE_DEBUG_ON_ERROR = False
pass_count = 0
fail_count = 0
skip_count = 0


def setup_debug_on_error(userdata):
    global BEHAVE_DEBUG_ON_ERROR
    BEHAVE_DEBUG_ON_ERROR = userdata.getbool("BEHAVE_DEBUG_ON_ERROR")


def before_all(context):
    logger = TestLog().set_logger(log_path=os.environ["test_log_file"],
                                  logging_level=os.environ["logging_level"], log_name="api automation")
    endpoint_url = os.environ['vcd-site_ose_url']
    access_key = eval(os.environ['vcd-site_ossauto1_tu1'])['access_key']
    secret_key = eval(os.environ['vcd-site_ossauto1_tu1'])['secret_key']

    boto3_client = Boto3Client()
    context.boto3_client = boto3_client
    logger.debug("create the instance of boto3 client")
    context.s3_client = boto3_client.get_boto3_client(endpoint=endpoint_url, access_key=access_key,
                                                      secret_key=secret_key)
    logger.debug("create the instance of boto3 resource")
    context.s3_resource = boto3_client.get_boto3_resource(endpoint=endpoint_url, access_key=access_key,
                                                          secret_key=secret_key)
    context.file_utils = FileUtils()
    context.logger = logger


def before_feature(context, feature):
    random_string = DataGenerator().id_generator_lowercase(size=10)
    context.bucket_name = "auto-api-test-" + random_string
    context.random_string = random_string


def after_feature(context, feature):
    if feature.status == Status.failed:
        context.logger.warning("======> Failing Feature: " + feature.name + "<======")
    else:
        context.logger.info("======> Passing Feature: " + feature.name + " <======")


def after_scenario(context, scenario):
    if scenario.status == Status.failed:
        context.logger.warning("======> Failing Scenario: " + scenario.name + "<======")
    else:
        context.logger.info("======> Passing Scenario: " + scenario.name + " <======")


def after_step(context, step):
    if step.status == Status.failed:
        context.logger.warning("======> Failing Step: " + step.name + "<======")


def after_all(context):
    pass
