@common_test_suite @common @copy_object
Feature: VMware OSE Common Test Suite/Validate Copy Object Functionality

  Scenario Outline: Copy object with Grant Access
    Given I am a tenant user <tenant_user>
    When I update object ACL to bucket with key <object_key> and parameters <parameters>
    Then I copy object to key: <object_key>
    Then I delete the source bucket with object key <object_key>
    Then I delete the destination bucket with object key <object_key>
    Examples: Upload object
      | tenant_user           | object_key        | parameters                                            |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"GrantFullControl" : "id={{vcd-site_ossauto1_tu1}}"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"GrantRead" : "id={{vcd-site_ossauto1_tu1}}"}        |
#      | vcd-site_ossauto1_tu1 | sample_object.txt | {"GrantReadACP" : "id={{vcd-site_ossauto2_tu2}}"}     |
#      | vcd-site_ossauto1_tu1 | sample_object.txt | {"GrantWriteACP" : "id={{vcd-site_ossauto2_tu2}}"}    |

  Scenario Outline: Copy object with CANNED ACL
    Given I am a tenant user <tenant_user>
    When I update object ACL to bucket with key <object_key> and parameters <parameters>
    Then I copy object to key: <object_key>
    Then I delete the source bucket with object key <object_key>
    Then I delete the destination bucket with object key <object_key>
    Examples: Upload object
      | tenant_user           | object_key        | parameters                            |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "private"}                   |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "public-read"}               |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "public-read-write"}         |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "authenticated-read"}        |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "bucket-owner-read"}         |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "bucket-owner-full-control"} |

  Scenario Outline: Copy object with different data (storage class/tagging/metadate/content-type...)
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I copy object from source bucket with key <object_key> and parameters <copy_parameters>
    Then I delete the source bucket with object key <object_key>
    Then I delete the destination bucket with object key <object_key>
    Examples: Copy Object
      | tenant_user           | object_key        | parameters                                      | copy_parameters                   |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"StorageClass":"STANDARD"}                     | {"CopySourceIfMatch":"{{e_tag}}"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "k1=v1"}                            | {"CopySourceIfMatch":"{{e_tag}}"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Metadata": {"metadata_key":"metadata_value"}} | {"CopySourceIfMatch":"{{e_tag}}"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ContentType" : "text/plain;charset=utf-8"}    | {"CopySourceIfMatch":"{{e_tag}}"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ContentLanguage" : "en-US"}                   | {"CopySourceIfMatch":"{{e_tag}}"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ContentEncoding" : "gzip"}                    | {"CopySourceIfMatch":"{{e_tag}}"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ContentDisposition" : "inline"}               | {"CopySourceIfMatch":"{{e_tag}}"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"CacheControl" : "CacheControl"}               | {"CopySourceIfMatch":"{{e_tag}}"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Expires" : "datetime.date(2026,1,1)"}         | {"CopySourceIfMatch":"{{e_tag}}"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"RequestPayer" : "test"}                       | {"CopySourceIfMatch":"{{e_tag}}"} |


  Scenario Outline: Copy object with CopySourceIfMatch (matched)
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I copy object from source bucket with key <object_key> and parameters <copy_parameters>
    Then I delete the source bucket with object key <object_key>
    Then I delete the destination bucket with object key <object_key>
    Examples: Copy Object
      | tenant_user           | object_key        | parameters                                      | copy_parameters                   |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Metadata": {"metadata_key":"metadata_value"}} | {"CopySourceIfMatch":"{{e_tag}}"} |

  Scenario Outline: Copy object with CopySourceIfMatch (non matched)
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I should fail to copy object from source bucket with key <object_key> and parameters <copy_parameters> when non matched found
    Then I verify expected error <expected_error> is found
    Then I delete the source bucket with object key <object_key>
    Then I delete the destination bucket with object key <object_key>
    Examples: Copy Object
      | tenant_user           | object_key        | parameters                                      | copy_parameters                    | expected_error                                                                                         |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Metadata": {"metadata_key":"metadata_value"}} | {"CopySourceIfMatch":"wrong_etag"} | An error occurred (412 Precondition Failed) when calling the CopyObject operation: Precondition Failed |

  Scenario Outline: Copy object with CopySourceIfNoneMatch (non matched)
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I copy object from source bucket with key <object_key> and parameters <copy_parameters>
    Then I delete the source bucket with object key <object_key>
    Then I delete the destination bucket with object key <object_key>
    Examples: Copy Object
      | tenant_user           | object_key        | parameters                                      | copy_parameters                       |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Metadata": {"metadata_key":"metadata_value"}} | {"CopySourceIfNoneMatch":"NoneMatch"} |


  Scenario Outline: Copy object with CopySourceIfNoneMatch (matched)
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I should fail to copy object from source bucket with key <object_key> and parameters <copy_parameters> when matched pre-condition found
    Then I verify expected error <expected_error> is found
    Then I delete the source bucket with object key <object_key>
    Then I delete the destination bucket with object key <object_key>
    Examples: Copy Object
      | tenant_user           | object_key        | parameters                                      | copy_parameters                       | expected_error                                                              |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Metadata": {"metadata_key":"metadata_value"}} | {"CopySourceIfNoneMatch":"{{e_tag}}"} | An error occurred (304) when calling the CopyObject operation: Not Modified |


  Scenario Outline: Copy object with CopySourceIfModifiedSince (matched)
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I copy object from source bucket with key <object_key> and parameters <copy_parameters>
    Then I delete the source bucket with object key <object_key>
    Then I delete the destination bucket with object key <object_key>
    Examples: Copy Object
      | tenant_user           | object_key        | parameters                                      | copy_parameters                                                                                             |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Metadata": {"metadata_key":"metadata_value"}} | {"CopySourceIfModifiedSince":"datetime.datetime.now(tz=datetime.timezone.utc)+datetime.timedelta(days=-1)"} |

  Scenario Outline: Copy object with CopySourceIfModifiedSince (non matched)
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I should fail to copy object from source bucket with key <object_key> and parameters <copy_parameters> when non matched found
    Then I verify expected error <expected_error> is found
    Then I delete the source bucket with object key <object_key>
    Then I delete the destination bucket with object key <object_key>
    Examples: Copy Object
      | tenant_user           | object_key        | parameters                                      | copy_parameters                                                                                              | expected_error                                                              |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Metadata": {"metadata_key":"metadata_value"}} | {"CopySourceIfModifiedSince":"datetime.datetime.now(tz=datetime.timezone.utc)+datetime.timedelta(days=365)"} | An error occurred (304) when calling the CopyObject operation: Not Modified |

  Scenario Outline: Copy object with CopySourceIfUnmodifiedSince (matched)
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I copy object from source bucket with key <object_key> and parameters <copy_parameters>
    Then I delete the source bucket with object key <object_key>
    Then I delete the destination bucket with object key <object_key>
    Examples: Copy Object
      | tenant_user           | object_key        | parameters                                      | copy_parameters                                                                                                |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Metadata": {"metadata_key":"metadata_value"}} | {"CopySourceIfUnmodifiedSince":"datetime.datetime.now(tz=datetime.timezone.utc)+datetime.timedelta(days=365)"} |


  Scenario Outline: Copy object with CopySourceIfUnmodifiedSince (non matched)
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I should fail to copy object from source bucket with key <object_key> and parameters <copy_parameters> when non matched found
    Then I verify expected error <expected_error> is found
    Then I delete the source bucket with object key <object_key>
    Then I delete the destination bucket with object key <object_key>
    Examples: Copy Object
      | tenant_user           | object_key        | parameters                                      | copy_parameters                                                                                               | expected_error                                                                                         |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Metadata": {"metadata_key":"metadata_value"}} | {"CopySourceIfUnmodifiedSince":"datetime.datetime.now(tz=datetime.timezone.utc)+datetime.timedelta(days=-1)"} | An error occurred (412 Precondition Failed) when calling the CopyObject operation: Precondition Failed |