@common_test_suite @common @object_basic
Feature: VMware OSE Common Test Suite/Validate Bucket Object Basic functionality

  Scenario Outline: Put object to own bucket
    Given I am a tenant user <tenant_user>
    When I upload object with key <object_key> to bucket <bucket_name>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | bucket_name | object_key        |
      | vcd-site_ossauto1_tu1 | api-test    | sample_object.txt |

  Scenario Outline: Put object to non-existing bucket
    Given I am a tenant user <tenant_user>
    When I upload object with key <object_key> to non-existing bucket
    Then I validate error <expected_status_code> when upload object to non-existing bucket
    Examples: Upload object
      | tenant_user           | object_key        | expected_status_code |
      | vcd-site_ossauto1_tu1 | sample_object.txt | 404                  |

  Scenario Outline: Put object without body
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key        | parameters  |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Body":""} |

  @debug_object_acl
  Scenario Outline: Put object with ACL
    Given I am a tenant user <tenant_user>
    When I update object ACL to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Change object ACL
      | tenant_user           | object_key        | parameters                            |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "private"}                   |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "public-read"}               |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "public-read-write"}         |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "authenticated-read"}        |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "bucket-owner-read"}         |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ACL" : "bucket-owner-full-control"} |

  @debug_object_acl
  Scenario Outline: Put object with Grant Access
    Given I am a tenant user <tenant_user>
    When I update object ACL to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Change object ACL
      | tenant_user           | object_key        | parameters                                            |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"GrantFullControl" : "id={{vcd-site_ossauto2_tu1}}"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"GrantRead" : "id={{vcd-site_ossauto1_tu1}}"}        |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"GrantReadACP" : "id={{vcd-site_ossauto1_tu1}}"}     |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"GrantWriteACP" : "id={{vcd-site_ossauto1_tu1}}"}    |

  Scenario Outline: Put object with Content Type
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key        | parameters                                   |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ContentType" : "text/plain;charset=utf-8"} |

  Scenario Outline: Put object with Metadata
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key        | parameters                |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Metadata": {"k1":"v1"}} |

  Scenario Outline: Put object with ContentDisposition
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key        | parameters                      |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ContentDisposition" : "TODO"} |

  Scenario Outline: Put object with Tagging
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key        | parameters           |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "k1=v1"} |

  Scenario Outline: Put object with ContentLanguage
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key        | parameters                   |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ContentLanguage" : "TODO"} |

  Scenario Outline: Put object with ContentEncoding
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key        | parameters                   |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"ContentEncoding" : "TODO"} |

  Scenario Outline: Put object with CacheControl
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key        | parameters                |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"CacheControl" : "TODO"} |


  Scenario Outline: Put folder with Content Type
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key | parameters                                   |
      | vcd-site_ossauto1_tu1 | folder/    | {"ContentType" : "text/plain;charset=utf-8"} |

  Scenario Outline: Put folder with Metadata
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key | parameters                |
      | vcd-site_ossauto1_tu1 | folder/    | {"Metadata": {"k1":"v1"}} |

  Scenario Outline: Put folder with ContentDisposition
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key | parameters                      |
      | vcd-site_ossauto1_tu1 | folder/    | {"ContentDisposition" : "TODO"} |

  Scenario Outline: Put folder with ContentLanguage
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key | parameters                   |
      | vcd-site_ossauto1_tu1 | folder/    | {"ContentLanguage" : "TODO"} |

  Scenario Outline: Put folder with ContentEncoding
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key | parameters                   |
      | vcd-site_ossauto1_tu1 | folder/    | {"ContentEncoding" : "TODO"} |

  Scenario Outline: Put folder with CacheControl
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key | parameters                |
      | vcd-site_ossauto1_tu1 | folder/    | {"CacheControl" : "TODO"} |

  Scenario Outline: Put folder with RequestPayer
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key | parameters                |
      | vcd-site_ossauto1_tu1 | folder/    | {"RequestPayer" : "TODO"} |

  Scenario Outline: Put object with RequestPayer
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key        | parameters                |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"RequestPayer" : "TODO"} |

  Scenario Outline: Put object with StorageClass
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key        | parameters                  |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"StorageClass":"STANDARD"} |

  Scenario Outline: Put object with empty key
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key | parameters             |
      | vcd-site_ossauto1_tu1 | ""         | {"Body":"sample test"} |

  Scenario Outline: Put object with Path Object
    Given I am a tenant user <tenant_user>
    When I upload Path object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key     | parameters               |
      | vcd-site_ossauto1_tu1 | sub1/sub2/obj2 | {"Body" : "sample test"} |

  Scenario Outline: Put object without body - path obj
    Given I am a tenant user <tenant_user>
    When I upload Path object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key     | parameters               |
      | vcd-site_ossauto1_tu1 | sub1/sub2/obj2 | {"Body" : "sample test"} |

  Scenario Outline: Put object to a nonexistent bucket - path obj
    Given I am a tenant user <tenant_user>
    When I upload object with key <object_key> to non-existing bucket
    Then I validate error <expected_status_code> when upload object to non-existing bucket
    Examples: Upload object
      | tenant_user           | object_key     | expected_status_code |
      | vcd-site_ossauto1_tu1 | sub1/sub2/obj2 | 404                  |

  Scenario Outline: Put object with empty obj name - path obj
    Given I am a tenant user <tenant_user>
    When I upload Path object to bucket with key <object_key> and parameters <parameters>
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key | parameters               |
      | vcd-site_ossauto1_tu1 | sub1/sub2/ | {"Body" : "sample test"} |

  Scenario Outline: Put object with deep path |folder
    Given I am a tenant user <tenant_user>
    When I upload Path object to bucket with deep path and parameters <parameters>
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | parameters               |
      | vcd-site_ossauto1_tu1 | {"Body" : "sample test"} |

  Scenario Outline: Put object with Path Object
    Given I am a tenant user <tenant_user>
    When I upload object with correct MD5 with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Upload object
      | tenant_user           | object_key     | parameters               |
      | vcd-site_ossauto1_tu1 | sub1/sub2/obj2 | {"Body" : "sample test"} |

  Scenario Outline: Put object with invalid MD5
    Given I am a tenant user <tenant_user>
    When I upload object with incorrect MD5 with key <object_key> and parameters <parameters>
    Examples: Upload object
      | tenant_user           | object_key     | parameters               |
      | vcd-site_ossauto1_tu1 | sub1/sub2/obj2 | {"Body" : "sample test"} |