@common_test_suite @common @object_tagging
Feature: VMware OSE Common Test Suite/Validate Object Tagging Functionality

  Scenario Outline: Put single object tagging with normal characters
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Object tagging
      | tenant_user           | object_key        | parameters           |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "k1=v1"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "K1=V1"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "1=1"}   |

  Scenario Outline: Put Single Object Tagging with special characters
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Object tagging
      | tenant_user           | object_key        | parameters             |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "k+1=v+1"} |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "k@1=v@1"} |


  Scenario Outline: Put object tag with correct content-MD5
    Given I am a tenant user <tenant_user>
    When I upload object tagging with correct MD5 with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Object tagging
      | tenant_user           | object_key        | parameters         |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {'key1': 'value1'} |

  Scenario Outline: Put object tag with incorrect content-MD5
    Given I am a tenant user <tenant_user>
    When I upload object with incorrect MD5 with key <object_key> and parameters <parameters>
    Then I verify expected error <expected_error> is found
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Object tagging
      | tenant_user           | object_key        | parameters                 | expected_error                                                     |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "key1=value1"} | An error occurred (BadDigest) when calling the PutObject operation |

  Scenario Outline: Put multiple object tagging
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Object tagging
      | tenant_user           | object_key        | parameters                                   |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "k1=value1&K2=value2&K1=value3"} |

  Scenario Outline: Put object tagging with content type
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Object tagging
      | tenant_user           | object_key        | parameters                                                             |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "key1=value2", "ContentType" : "text/plain;charset=utf-8"} |


  Scenario Outline: Put object tagging with RequestPayer
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Object tagging
      | tenant_user           | object_key        | parameters                                               |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "key1=value2", "RequestPayer" : "requester"} |


  @debug_object_tagging
  Scenario Outline: Get object tagging
    Given I am a tenant user <tenant_user>
    When I upload object to bucket with key <object_key> and parameters <parameters>
    Then I get object tagging with key <object_key>
    Then I delete object with key <object_key> from bucket
    Then I delete the bucket
    Examples: Object tagging
      | tenant_user           | object_key        | parameters           |
      | vcd-site_ossauto1_tu1 | sample_object.txt | {"Tagging": "k1=v1"} |