import json
import os

from behave import *
from common_libraries.boto3_client import Boto3Client
from common_libraries.data_providers.file_utils import FileUtils
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog
from testbase.api_testbase import ApiTestBase

api_testBase = ApiTestBase()


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Given("I am a tenant user {tenant_user}")
def step_impl(context, tenant_user):
    endpoint_url = ""
    access_key = ""
    secret_key = ""
    try:
        if 'vcd-site_ose_url' in os.environ:
            endpoint_url = os.environ['vcd-site_ose_url']
        else:
            endpoint_url = None
        access_key = eval(os.environ[tenant_user])['access_key']
        secret_key = eval(os.environ[tenant_user])['secret_key']
        tenant_user_name = eval(os.environ[tenant_user])['username']
        print(str(os.environ))
        context.logger.debug(str(os.environ))
        log_message = "Connect to S3 client information: Tenant user name: " + tenant_user_name + ", S3 endpoint url: " + endpoint_url + ", S3 access key: " + access_key + " and S3 secret key: " + secret_key
        context.logger.debug(log_message)
        context.s3_client = Boto3Client().get_boto3_client(endpoint=endpoint_url, access_key=access_key,
                                                           secret_key=secret_key)
        context.s3_resource = Boto3Client().get_boto3_resource(endpoint=endpoint_url, access_key=access_key,
                                                               secret_key=secret_key)
        TestLog.attach_allure_log(
            log_message="Connect to S3 client with S3 user: " + tenant_user + ", S3 endpoint url: " + endpoint_url + ", S3 access key: " + access_key + " and S3 secret key: " + secret_key,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except Exception as ex:
        log_message = "Fail to connect to S3 client with S3 user: " + tenant_user + ", S3 endpoint url: " + endpoint_url + ", S3 access key: " + access_key + " and S3 secret key: " + secret_key
        TestLog.attach_allure_log(
            log_message="Fail to connect to S3 client with S3 user: " + tenant_user + ", S3 endpoint url: " + endpoint_url + ", S3 access key: " + access_key + " and S3 secret key: " + secret_key,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        TestLog.attach_allure_log(
            log_message="Fail to connect to S3 client and caused by the exception '" + str(ex) + "'",
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_message)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Given("I am the other tenant user {other_tenant_user}")
def step_impl(context, other_tenant_user):
    endpoint_url = ""
    access_key = ""
    secret_key = ""
    try:
        endpoint_url = os.environ['vcd-site_ose_url']
        access_key = eval(os.environ[other_tenant_user])['access_key']
        secret_key = eval(os.environ[other_tenant_user])['secret_key']
        tenant_user_name = eval(os.environ[other_tenant_user])['username']
        log_message = "Connect to S3 client information: Tenant user name: " + tenant_user_name + ", S3 endpoint url: " + endpoint_url + ", S3 access key: " + access_key + " and S3 secret key: " + secret_key
        context.logger.debug(log_message)
        context.s3_client = Boto3Client().get_boto3_client(endpoint=endpoint_url, access_key=access_key,
                                                           secret_key=secret_key)
        context.s3_resource = Boto3Client().get_boto3_resource(endpoint=endpoint_url, access_key=access_key,
                                                               secret_key=secret_key)
        TestLog.attach_allure_log(
            log_message="Connect to S3 client with S3 user: " + other_tenant_user + ", S3 endpoint url: " + endpoint_url + ", S3 access key: " + access_key + " and S3 secret key: " + secret_key,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except Exception as ex:
        log_message = "Fail to connect to S3 client with S3 user: " + other_tenant_user + ", S3 endpoint url: " + endpoint_url + ", S3 access key: " + access_key + " and S3 secret key: " + secret_key
        TestLog.attach_allure_log(
            log_message="Fail to connect to S3 client with S3 user: " + other_tenant_user + ", S3 endpoint url: " + endpoint_url + ", S3 access key: " + access_key + " and S3 secret key: " + secret_key,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        TestLog.attach_allure_log(
            log_message="Fail to connect to S3 client and caused by the exception '" + str(ex) + "'",
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_message)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Given("Get x-vcloud-authorization of user name {user_name} and password {password} on VCD site {vcd_endpoint_url}")
def step_impl(context, user_name, password, vcd_endpoint_url):
    basic_auth = None
    req_header = None
    if user_name and password:
        user_pass = user_name + ':' + password
        basic_auth = api_testBase.decode_base64_data(user_pass)
    if basic_auth:
        req_header = {
            'Accept': 'application/*+json;version=33.0,application/json;version=33.0',
            'Authorization': 'Basic ' + basic_auth + ''
        }
    try:
        context.logger.debug("vcd_endpoint: " + str(os.environ[vcd_endpoint_url]))
        os.environ["vcd_site"] = str(vcd_endpoint_url).split("_")[0]
        context.logger.debug("vcd_site: " + str(os.environ["vcd_site"]))
        x_vcloud_authorization, status_code = api_testBase.get_vcloud_authorization(
            vcd_endpoint_url=os.environ[vcd_endpoint_url], req_header=req_header)
        # Store vcloud authorization
        os.environ["x_vcloud_authorization"] = x_vcloud_authorization
        context.logger.debug("x_vcloud_authorization: " + x_vcloud_authorization)
    except Exception as ex:
        context.logger.debug(ex)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I Verify Http Status code {expected_status_code} in the response")
def step_impl(context, expected_status_code):
    try:
        actual_status_code = os.environ['http_status_code']
        api_testBase.verify_http_status_code(int(actual_status_code), int(expected_status_code))
    except Exception as ex:
        raise Exception("Verify Http Status code throw the exception: " + ex)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I Verify Http Status code {delete_bucket_status_code} in DELETE Bucket response")
def step_impl(context, delete_bucket_status_code):
    try:
        actual_status_code = os.environ['http_status_code']
        api_testBase.verify_http_status_code(int(actual_status_code), int(delete_bucket_status_code))
    except Exception as ex:
        raise Exception("Verify Http Status code throw the exception: " + ex)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I Verify Http Status code {create_bucket_status_code} in Create Bucket response")
def step_impl(context, create_bucket_status_code):
    try:
        actual_status_code = os.environ['http_status_code']
        api_testBase.verify_http_status_code(int(actual_status_code), int(create_bucket_status_code))
    except Exception as ex:
        raise Exception("Verify Http Status code throw the exception: " + ex)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("Load data from external csv file {csv_file_name}")
def step_impl(context, csv_file_name):
    test_data_directory = os.getenv("test_data_directory")
    test_data = FileUtils.load_csv(os.path.join(test_data_directory, csv_file_name))
    context.logger.debug("test data: " + test_data)
    context.data = test_data


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("Load data from external excel file {excel_file_name} with sheet name {sheet_name}")
def step_impl(context, excel_file_name, sheet_name):
    test_data_directory = os.getenv("test_data_directory")
    test_data = FileUtils.load_xlsx(excel_file_path=os.path.join(test_data_directory, excel_file_name),
                                    sheet_name=sheet_name)
    context.data = test_data


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("Load data from external yaml file")
def step_impl(context):
    yaml_file_path = os.getenv("test_environment_yaml")
    environment_data = FileUtils.load_yaml_with_full_loader(yaml_file_path)
    print(str(environment_data))
    context.environment_data = str(environment_data)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then(
    "Get x-vcloud-authorization of specific user {authorization_user} on VCD site {vcd_endpoint_url} and Tenant {tenant}")
def step_impl(context, vcd_endpoint_url, authorization_user, tenant):
    test_data_list = context.data[0]
    environment_data = context.environment_data
    user_name = eval(environment_data)[str(test_data_list['HttpAuthorizationUser']).replace("{{", "").replace("}}", "")]
    password = eval(environment_data)[
        str(test_data_list['HttpAuthorizationPassword']).replace("{{", "").replace("}}", "")]
    try:
        if user_name and password:
            user_pass = user_name + ':' + password
            basic_auth = api_testBase.decode_base64_data(user_pass)
            req_header = {
                'Accept': 'application/*+json;version=35.2,application/json;version=35.2',
                'Authorization': 'Basic ' + basic_auth + ''
            }
        os.environ["vcd_site"] = str(vcd_endpoint_url).split("_")[0]
        x_vcloud_authorization, status_code = api_testBase.get_vcloud_authorization(
            vcd_endpoint_url=os.environ[vcd_endpoint_url], req_header=req_header)
        # Store vcloud authorization
        os.environ["x_vcloud_authorization"] = x_vcloud_authorization
        context.logger.debug("x_vcloud_authorization: " + x_vcloud_authorization)
    except Exception as ex:
        context.logger.debug(ex)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("Send Http Request and Verify Http response")
def step_impl(context):
    try:
        test_data_dict = context.data[0]
        environment_data = eval(context.environment_data)
        test_case_title = test_data_dict['Test Case #']
        http_action, http_request_url, http_request_header, http_request_body = api_testBase.prepare_http_request_data(
            str(test_data_dict), environment_data)
        # send http request
        http_response = api_testBase.send_http_request(http_action=http_action, http_request_url=http_request_url,
                                                       http_request_body=http_request_body,
                                                       http_request_header=http_request_header)
        # verify http response
        if http_response:
            if 'ExpectResponseCode' in test_data_dict:
                expected_response_code = test_data_dict['ExpectResponseCode']
                verify_result = api_testBase.verify_http_status_code(actual_status_code=http_response.status_code,
                                                                     expected_status_code=expected_response_code)
                if verify_result:
                    context.logger.debug("----- success to verify http status code -----")
                else:
                    raise Exception("Verify http status code failed !")
            context.logger.debug("http response: " + str(http_response.text))
            if 'OutputParameter' in test_data_dict:
                output_parma = test_data_dict['OutputParameter']
                data = json.loads(str(http_response.text))
                environment_data[output_parma] = data[output_parma]
                context.logger.debug(environment_data[output_parma])

        else:
            raise Exception("Http response is not returned as expected !")

        context.logger.debug("----- end to run test case " + test_case_title + " -----")

    except Exception as ex:
        raise Exception("Fail to send the http request '" + ex + "' and check the https status code !")
