import os
import re
from time import sleep

from behave import *
from botocore.exceptions import ClientError

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog
from testbase.teamplates.bucket_acl_template import BucketAclTemplate


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When(
    "I Put Bucket ACL to tenant user {owner_display_name},{owner_id},{grantee_display_name},{grantee_id},{grantee_type},{grantee_permission}")
def step_impl(context, owner_display_name, owner_id, grantee_display_name, grantee_id, grantee_type,
              grantee_permission):
    """
    Put Bucket ACL to specified tenant user
    :param context:
    :param owner_display_name:
    :param owner_id:
    :param grantee_display_name:
    :param grantee_id:
    :param grantee_type:
    :param grantee_permission:
    :return:
    """
    if context.bucket_name:
        bucket_name = context.bucket_name
    else:
        bucket_name = DataGenerator().generate_bucket_name(context)
        os.environ["bucket_name"] = bucket_name
        context.bucket_name = bucket_name
    try:
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            owner_display_name = eval(os.environ[owner_display_name])['canonical_id']
            owner_id = eval(os.environ[owner_id])['canonical_id']
            grantee_display_name = eval(os.environ[grantee_display_name])['canonical_id']
            grantee_id = eval(os.environ[grantee_id])['canonical_id']
            parameters = {"owner_display_name": owner_display_name, "owner_id": owner_id,
                          "grantee_display_name": grantee_display_name, "grantee_id": grantee_id,
                          "grantee_type": grantee_type, "grantee_permission": grantee_permission}

            bucket_acl = BucketAclTemplate.get_bucket_access_control_policy(**parameters)
            context.logger.debug("bucket acl: " + str(bucket_acl))
            response = context.s3_resource.BucketAcl(bucket_name).put(
                AccessControlPolicy=bucket_acl
            )
            if response:
                http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
                log_message = "Add Bucket ACL: " + str(bucket_acl) + " and get the status code: " + str(
                    http_status_code)
                context.logger.debug(log_message)
                TestLog.attach_allure_log(
                    log_message=log_message,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                context.actual_status_code_create_bucket = http_status_code
    except ClientError as ex:
        log_error = "Fail to Put Bucket ACL for the bucket '" + bucket_name + "' and get the unexpected exception " + str(
            ex)
        context.logger.error(log_error, exc_info=True)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When(
    "I Put Bucket ACL to All Users:{uri}, {owner_id},{grantee_id},{grantee_type},{grantee_permission}")
def step_impl(context, uri, owner_id, grantee_id, grantee_type,
              grantee_permission):
    """
    Put ACL to all authenticated user
    :param context:
    :param uri:
    :param owner_id:
    :param grantee_id:
    :param grantee_type:
    :param grantee_permission:
    :return:
    """

    bucket_name = DataGenerator().generate_bucket_name(context)
    try:
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            # get value from global environment variables
            owner_display_name = eval(os.environ[owner_id])['canonical_id']
            owner_id = eval(os.environ[owner_id])['canonical_id']
            grantee_display_name = eval(os.environ[grantee_id])['canonical_id']
            grantee_id = eval(os.environ[grantee_id])['canonical_id']
            parameters = {"uri": uri, "owner_display_name": owner_display_name, "owner_id": owner_id,
                          "grantee_display_name": grantee_display_name, "grantee_id": grantee_id,
                          "grantee_type": grantee_type, "grantee_permission": grantee_permission}

            bucket_acl = BucketAclTemplate.get_bucket_acl_for_all_users(**parameters)
            response = context.s3_resource.BucketAcl(bucket_name).put(
                AccessControlPolicy=bucket_acl
            )
            http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
            log_message = "Add Bucket ACL: " + str(bucket_acl) + " and get the status code: " + str(http_status_code)
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            context.actual_status_code_create_bucket = http_status_code

    except ClientError as ex:
        log_error = "Fail to Put Bucket ACL for the bucket '" + bucket_name + "' and get the unexpected exception " + str(
            ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When(
    "I Put Bucket ACL to Authenticated Users:{uri}, {owner_id},{grantee_id},{grantee_type},{grantee_permission}")
def step_impl(context, uri, owner_id, grantee_id, grantee_type,
              grantee_permission):
    if context.bucket_name:
        bucket_name = context.bucket_name
    else:
        bucket_name = DataGenerator().id_generator_lowercase(size=15)
        os.environ["bucket_name"] = bucket_name
        context.bucket_name = bucket_name
    try:
        if context.bucket_name:
            bucket_name = context.bucket_name
        else:
            bucket_name = DataGenerator().id_generator_lowercase(size=15)
            os.environ["bucket_name"] = bucket_name
            context.bucket_name = bucket_name
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            context.logger.debug("bucket exists.......")
        else:
            for i in range(3):
                sleep(1)
                context.logger.debug("wait for bucket created.......")
                if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
                    break
        # get value from global environment variables
        owner_display_name = eval(os.environ[owner_id])['canonical_id']
        context.logger.debug(owner_display_name)
        owner_id = eval(os.environ[owner_id])['canonical_id']
        context.logger.debug(owner_id)
        grantee_display_name = eval(os.environ[grantee_id])['canonical_id']
        context.logger.debug(grantee_display_name)
        grantee_id = eval(os.environ[grantee_id])['canonical_id']
        context.logger.debug(grantee_id)
        parameters = {"uri": uri, "owner_display_name": owner_display_name, "owner_id": owner_id,
                      "grantee_display_name": grantee_display_name, "grantee_id": grantee_id,
                      "grantee_type": grantee_type, "grantee_permission": grantee_permission}

        bucket_acl = BucketAclTemplate.get_bucket_acl_for_all_users(**parameters)
        context.logger.debug("bucket acl: " + str(bucket_acl))
        response = context.s3_resource.BucketAcl(bucket_name).put(
            AccessControlPolicy=bucket_acl
        )
        http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
        log_message = "Add Bucket ACL: " + str(bucket_acl) + " and get the status code: " + str(http_status_code)
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        context.actual_status_code_create_bucket = http_status_code

    except ClientError as ex:
        log_error = "Fail to Put Bucket ACL for the bucket '" + bucket_name + "' and get the unexpected exception " + str(
            ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I created a bucket and set ACL {acl}")
def step_impl(context, acl):
    bucket_name = DataGenerator.generate_bucket_name(context)
    http_status_code = context.boto3_client.s3_create_bucket(bucket_name=bucket_name, acl=acl)
    context.actual_status_code_create_bucket = http_status_code
    if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
        context.logger.debug("bucket exists.......")
    # create bucket
    context.logger.debug("create the bucket successfully : " + str(bucket_name))
    log_message = "Create the bucket and set ACL: " + str(bucket_name) + " and get the expected status code: " + str(
        http_status_code)
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I created a bucket and granted ACL {grant_acl}")
def step_impl(context, grant_acl):
    try:
        bucket_name = DataGenerator.generate_bucket_name(context)
        context.logger.debug("bucket name: " + context.bucket_name)
        # parse arguments
        kwargs = eval(grant_acl)
        for key, value in kwargs.items():
            if "Grant" in str(key):
                source_string = str(kwargs)
                variables_to_replace = re.findall("{{.*?}}", source_string)
                value = eval(os.environ[variables_to_replace[0].replace("{{", "").replace("}}", "")])['canonical_id']
                kwargs = eval(source_string.replace(variables_to_replace[0], value))
        grant_acl = str(kwargs)
        http_status_code = context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        bucket_acl = context.s3_resource.BucketAcl(bucket_name)
        bucket_acl.put(**eval(grant_acl))
        context.actual_status_code_create_bucket = http_status_code
        context.logger.debug("create bucket status: " + str(http_status_code))
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            context.logger.debug("bucket exists.......")
        # create bucket and grant ACL
        log_message = "Create the bucket and Grant ACL: " + str(
            bucket_name) + " And get the expected status code: " + str(
            http_status_code)
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        error_code = ex.response['Error']['Code']
        error_message = ex.response['Error']['Message']
        http_status_code = int(ex.response['ResponseMetadata']['HTTPStatusCode'])
        if 400 <= http_status_code < 500:
            log_error = "Fail to Create bucket with grant ACL, caused by client side error, error code '" + error_code + "' And '" + error_message + "'"
        elif http_status_code >= 500:
            log_error = "Fail to Create bucket with grant ACL, caused by server side error, error code '" + error_code + "' And '" + error_message + "'"
        else:
            log_error = "Fail to Create bucket with grant ACL, caused by other issue, error code '" + error_code + "' And error message '" + error_message + "'"
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When(
    "I Put Bucket ACL with incorrect content-MD5 check: {owner_display_name},{owner_id},{grantee_display_name},{grantee_id},{grantee_type},{grantee_permission}")
def step_impl(context, owner_display_name, owner_id, grantee_display_name, grantee_id, grantee_type,
              grantee_permission):
    if context.bucket_name:
        bucket_name = context.bucket_name
    else:
        bucket_name = DataGenerator.generate_bucket_name(context)
    try:
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            owner_display_name = eval(os.environ[owner_display_name])['canonical_id']
            owner_id = eval(os.environ[owner_id])['canonical_id']
            grantee_display_name = eval(os.environ[grantee_display_name])['canonical_id']
            grantee_id = eval(os.environ[grantee_id])['canonical_id']
            parameters = {"owner_display_name": owner_display_name, "owner_id": owner_id,
                          "grantee_display_name": grantee_display_name, "grantee_id": grantee_id,
                          "grantee_type": grantee_type, "grantee_permission": grantee_permission}
            bucket_acl = BucketAclTemplate.get_bucket_access_control_policy(**parameters)

            context.logger.debug("bucket acl: " + str(bucket_acl))

            md5_bucket_acl = DataGenerator().calculate_md5("wrong content-MD5")
            context.s3_resource.BucketAcl(bucket_name).put(
                AccessControlPolicy=bucket_acl, ContentMD5=md5_bucket_acl
            )

    except ClientError as ex:
        if "An error occurred (BadDigest) when calling the PutBucketAcl operation" in str(ex):
            log_error = "Put Bucket ACL with incorrect MD5 : " + str(
                bucket_name) + " and get the response: " + str(
                str(ex))
            context.logger.debug(log_error)
            TestLog.attach_allure_log(
                log_message=log_error,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When(
    "I Put Bucket ACL with empty ACL settings and get the expected status code:{expected_error_message}")
def step_impl(context, expected_error_message):
    try:
        if context.bucket_name:
            bucket_name = context.bucket_name
        else:
            bucket_name = DataGenerator.generate_bucket_name(context)
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            response = context.s3_resource.BucketAcl(bucket_name).put(
                AccessControlPolicy={}
            )
            context.logger.debug("Put bucket acl: " + str(response))
    except Exception as ex:
        if expected_error_message in str(ex):
            pass
        else:
            raise Exception("The expected error message '" + expected_error_message + "' is not returned!")


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then(
    "I Put Bucket Canned ACL to tenant user: {canned_acl}")
def step_impl(context, canned_acl):
    """
    Canned ACL could be: private, public-read,public write,public read-write,authenticated-read
    :param canned_acl:
    :param context:
    :return:
    """
    if context.bucket_name:
        bucket_name = context.bucket_name
    else:
        bucket_name = DataGenerator.generate_bucket_name(context)
    try:

        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            context.logger.debug("bucket acl: " + str(canned_acl))
            response = context.s3_resource.BucketAcl(bucket_name).put(
                ACL=canned_acl
            )
            http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
            context.actual_status_code_create_bucket = http_status_code
            log_message = "Put Bucket Canned ACL: " + str(
                bucket_name) + " and get the expected status code: " + str(
                http_status_code)
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)

    except ClientError as ex:
        error_code = ex.response['Error']['Code']
        error_message = ex.response['Error']['Message']
        http_status_code = int(ex.response['ResponseMetadata']['HTTPStatusCode'])
        if 400 <= http_status_code < 500:
            log_error = "Fail to Put Bucket Canned ACL for the bucket '" + bucket_name + "', caused by client side error, error code '" + error_code + "' And '" + error_message + "'"
        elif http_status_code >= 500:
            log_error = "Fail to Put Bucket Canned ACL for the bucket '" + bucket_name + " ', caused by server side error, error code '" + error_code + "' And '" + error_message + "'"
        else:
            log_error = "Fail to Put Bucket Canned ACL for the bucket '" + bucket_name + "', caused by other issue, error code '" + error_code + "' And error message '" + error_message + "'"
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(error_message)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I get bucket ACL")
def step_impl(context):
    bucket_name = context.bucket_name
    try:
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            # get bucket acl grants
            grants_response = context.s3_resource.BucketAcl(bucket_name).grants
            context.logger.debug("get bucket acl's grants_response: " + str(grants_response))
            # get bucket acl owner
            owner_response = context.s3_resource.BucketAcl(bucket_name).owner
            context.logger.debug("get bucket acl's owner_response: " + str(owner_response))
            TestLog.attach_allure_log(
                log_message="get bucket ACL and get grants: " + str(grants_response),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            TestLog.attach_allure_log(
                log_message="get bucket ACL and get owner: " + str(grants_response),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        else:
            raise Exception("Bucket '" + bucket_name + "' is not found as expected!")
    except ClientError as ex:
        error_code = ex.response['Error']['Code']
        error_message = ex.response['Error']['Message']
        http_status_code = int(ex.response['ResponseMetadata']['HTTPStatusCode'])
        if 400 <= http_status_code < 500:
            log_error = "Fail to get bucket ACL for the bucket '" + bucket_name + "', caused by client side error, error code '" + error_code + "' And error message '" + error_message + "'"
        elif http_status_code >= 500:
            log_error = "Fail to get bucket ACL for the bucket '" + bucket_name + "', caused by server side error, error code '" + error_code + "' And error message '" + error_message + "'"
        else:
            log_error = "Fail to get bucket ACL for the bucket '" + bucket_name + "', caused by other issue, error code '" + error_code + "' And error message '" + error_message + "'"
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(error_message)
