from behave import *
from botocore.exceptions import ClientError

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog
from common_test_suite.templates.bucket_cors import BucketCorsTemplate


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I Add bucket Cors to bucket with template: {bucket_cors_template}")
def step_impl(context, bucket_cors_template=None):
    """
    Add Bucket CORS
    :param context:
    :param bucket_cors_template:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context)
    try:
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        if bucket_cors_template:
            bucket_cors = BucketCorsTemplate.get_bucket_cors_template(bucket_cors_template)
        else:
            raise Exception("Bucket CORS Template is not found as expected!")
        if "{{bucket_name}}" in str(bucket_cors):
            bucket_cors = str(bucket_cors).replace("{{bucket_name}}", bucket_name)
        response = context.s3_resource.BucketCors(bucket_name).put(
            CORSConfiguration=bucket_cors
        )
        if response:
            http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
            TestLog.attach_allure_log(
                log_message="Add bucket cors rule: " + str(bucket_cors) + "and get status code: " + str(
                    http_status_code),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            context.actual_status_code_create_bucket = http_status_code
    except Exception as ex:
        raise Exception("Fail to add CORS to bucket '" + bucket_name + "'")


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I validate bucket cors with wrong argument and template: {bucket_cors_template}")
def step_impl(context, bucket_cors_template=None):
    """
    Validate Bucket CORS
    :param context:
    :param bucket_cors_template:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context)
    try:
        create_bucket_status_code = context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        context.actual_status_code_create_bucket = create_bucket_status_code

        if bucket_cors_template:
            cors_configuration = BucketCorsTemplate.get_bucket_cors_template(bucket_cors_template)
        else:
            raise Exception("Bucket CORS Template is not found as expected!")
        if "{{bucket_name}}" in str(cors_configuration):
            str(cors_configuration).replace("{{bucket_name}}", bucket_name)
        response = context.s3_resource.BucketCors(bucket_name).put(
            CORSConfiguration=cors_configuration, ContentMD5="12345654321"
        )
        http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
        context.actual_status_code_create_bucket = http_status_code

    except ClientError as ex:
        if 'The Content-MD5 you specified is not valid' in str(ex):
            log_message = "Validate bucket cors with wrong argument and template: " + str(
                bucket_name) + " and get the exception: " + str(
                ex)
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        else:
            log_exception = "Add Bucket CORS validation failed: " + str(ex)
            TestLog.attach_allure_log(
                log_message=log_exception,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
            raise Exception(log_exception)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I validate bucket cors with contentMD5: {bucket_cors_template}")
def step_impl(context, bucket_cors_template=None):
    """
    Validate Bucket CORS with invalid contentMD5
    :param context:
    :param bucket_cors_template:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context)
    try:
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        if bucket_cors_template:
            cors_configuration = BucketCorsTemplate.get_bucket_cors_template(bucket_cors_template)
        else:
            raise Exception("Bucket CORS Template is not found as expected!")

        if "{{bucket_name}}" in str(cors_configuration):
            str(cors_configuration).replace("{{bucket_name}}", bucket_name)
            context.logger.debug("bucket cors rule: " + str(cors_configuration))
        response = context.s3_resource.BucketCors(bucket_name).put(
            CORSConfiguration=cors_configuration, ContentMD5="12345654321"
        )
        http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
        context.actual_status_code_create_bucket = http_status_code
        # Add Bucket CORS
        context.logger.debug("Add Bucket CORS successfully : " + str(http_status_code))
        context.actual_status_code_create_bucket = http_status_code
        TestLog.attach_allure_log(
            log_message="Add bucket cors rule: " + str(cors_configuration) + "and get status code: " + str(
                http_status_code),
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        if 'The Content-MD5 you specified is not valid' in str(ex):
            pass
        else:
            raise ex


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I get bucket CORS")
def step_impl(context):
    """
    Get Bucket CORs
    :param context:
    :return:
    """
    try:
        get_bucket_cors_response = context.boto3_client.get_bucket_cors(
            bucket_name=context.bucket_name
        )
        log_message = "Get bucket CORs's response: " + str(get_bucket_cors_response)
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        if 'The Content-MD5 you specified is not valid' in str(ex):
            pass
        else:
            raise ex


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete bucket CORS")
def step_impl(context):
    """
    Delete Bucket CORS
    :param context:
    :return:
    """
    try:
        delete_bucket_cors_response = context.s3_resource.BucketCors(context.bucket_name).delete()
        TestLog.attach_allure_log(log_message=str(delete_bucket_cors_response),
                                  log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        if 'The Content-MD5 you specified is not valid' in str(ex):
            pass
        else:
            raise ex


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete bucket CORS and check the status code {bucket_cors_expected_status_code}")
def step_impl(context, bucket_cors_expected_status_code):
    """
    Delete Bucket CORS and verify the status
    :param context:
    :param bucket_cors_expected_status_code:
    :return:
    """
    try:
        response = context.s3_resource.BucketCors(context.bucket_name).delete()
        if str(response['ResponseMetadata']['HTTPStatusCode']) == bucket_cors_expected_status_code:
            TestLog.attach_allure_log(
                log_message="Excepted status code [" + bucket_cors_expected_status_code + "] is found as expected",
                log_type=TestLog.LOG_TYPE_TEST_VALIDATION)
        else:
            TestLog.attach_allure_log(
                log_message="Excepted status code [" + bucket_cors_expected_status_code + "] is NOT found as expected",
                log_type=TestLog.LOG_TYPE_TEST_VALIDATION)
            raise Exception("Excepted status code [" + bucket_cors_expected_status_code + "] is NOT found as expected")

    except ClientError as ex:
        if 'The Content-MD5 you specified is not valid' in str(ex):
            pass
        else:
            raise ex


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I validate bucket CORs with expected message {expected_message}")
def step_impl(context, expected_message):
    """
    Get Bucket CORS and verify that the expected messages is found as expected
    :param context:
    :param expected_message:
    :return:
    """
    try:
        response = context.boto3_client.get_bucket_cors(bucket_name=context.bucket_name)
        for res in response:
            context.logger.debug("actual response: " + str(res))
            if str(expected_message) in str(res):
                validation_message = "Expected message [" + expected_message + "] is found in the response [" + str(
                    response[0]) + "] as expected."
                TestLog.attach_allure_log(log_message=validation_message, log_type=TestLog.LOG_TYPE_TEST_VALIDATION)
                return True
            else:
                error_message = "Expected message [" + expected_message + "] is NOT found in the response [" + str(
                    response[0]) + "] as expected!"
                TestLog.attach_allure_log(log_message=error_message, log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
                raise Exception(error_message)

    except ClientError as ex:
        if 'The Content-MD5 you specified is not valid' in str(ex):
            pass
        else:
            raise ex
