import os

from behave import *
from botocore.exceptions import ClientError

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog
from common_test_suite.templates.bucket_lifecycle import BucketLifecycleTemplate


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I Add Bucket Lifecycle for all objects")
def step_impl(context):
    """
    Add Bucket Lifecycle
    :param context:
    :return:
    """
    try:
        bucket_name = DataGenerator.generate_bucket_name(context)
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        bucket_lifecycle_rule = BucketLifecycleTemplate.get_bucket_lifecycle_for_all_object()
        response = context.s3_resource.BucketLifecycleConfiguration(bucket_name).put(
            LifecycleConfiguration=bucket_lifecycle_rule)
        http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
        context.actual_status_code_put_bucket_lifecycle = http_status_code
        context.put_bucket_lifecycle_rule = eval(bucket_lifecycle_rule)
        log_message = "Add Bucket Lifecycle: " + str(
            bucket_name) + " and get the expected status code: " + str(
            http_status_code)
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except Exception as ex:
        raise Exception("Fail to add Bucket Lifecycle: " + str(ex))


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I Add Bucket Lifecycle: {lifecycle_rule} with prefix")
def step_impl(context, lifecycle_rule=None):
    bucket_name = DataGenerator.generate_bucket_name(context)
    logger = context.logger
    try:

        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        bucket_lifecycle_rule = BucketLifecycleTemplate.get_bucket_lifecycle_with_prefix(**eval(lifecycle_rule))
        http_status_code = context.s3_resource.BucketLifecycle(bucket_name).put(
            LifecycleConfiguration=bucket_lifecycle_rule)
        context.actual_status_code_put_bucket_lifecycle = http_status_code
        context.put_bucket_lifecycle_rule = bucket_lifecycle_rule
        log_message = "Add Bucket Lifecycle: " + str(
            bucket_name) + " and get the expected status code: " + str(
            http_status_code)
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        error_message = "Add Bucket Lifecycle with prefix and get the unexpected error: " + \
                        ex.response['Error']['Message']
        logger.error(error_message, exc_info=True)
        TestLog.attach_allure_log(
            log_message=error_message,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(error_message)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I Add Bucket Lifecycle: {lifecycle_rule} with filter")
def step_impl(context, lifecycle_rule=None):
    """
    Add Bucket Lifecycle with filter
    :param context:
    :param lifecycle_rule:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context)
    logger = context.logger
    try:

        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        bucket_lifecycle_rule = BucketLifecycleTemplate.get_bucket_lifecycle_with_filter(**eval(lifecycle_rule))
        http_status_code = context.s3_resource.BucketLifecycle(bucket_name).put(
            LifecycleConfiguration=bucket_lifecycle_rule)
        context.actual_status_code_put_bucket_lifecycle = http_status_code
        context.put_bucket_lifecycle_rule = bucket_lifecycle_rule
        log_message = "Add Bucket Lifecycle: " + str(
            bucket_name) + " and get the expected status code: " + str(
            http_status_code)
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        error_message = "Add Bucket Lifecycle with filter and get the unexpected error: " + \
                        ex.response['Error']['Message']
        logger.error(error_message, exc_info=True)
        TestLog.attach_allure_log(
            log_message=error_message,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(error_message)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I get Bucket lifecycle")
def step_impl(context):
    """
    Get Bucket Lifecycle
    :param context:
    :return:
    """
    bucket_name = os.environ["bucket_name"]
    bucket_lifecycle_rule = context.s3_resource.BucketLifecycle(bucket_name).rules
    context.get_bucket_lifecycle_rule = bucket_lifecycle_rule
    if 'put_bucket_lifecycle_rule' in context and context.put_bucket_lifecycle_rule:
        context.logger.debug(str(bucket_lifecycle_rule) in str(context.put_bucket_lifecycle_rule))
    log_message = "Get Bucket Lifecycle: " + str(
        bucket_name) + " and get the expected result: " + str(
        bucket_lifecycle_rule)
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I validate Bucket lifecycle")
def step_impl(context):
    """
    Validate Bucket lifecycle
    :param context:
    :return:
    """
    bucket_name = os.environ["bucket_name"]
    bucket_lifecycle_rule = context.s3_resource.BucketLifecycle(bucket_name).rules
    context.logger.debug("get bucket lifecycle rule: " + str(bucket_lifecycle_rule))
    if 'put_bucket_lifecycle_rule' in context and context.put_bucket_lifecycle_rule:
        if str(bucket_lifecycle_rule) in str(context.put_bucket_lifecycle_rule):
            log_message = "Validate Bucket Lifecycle: " + str(
                bucket_name) + " and get the expected status code: " + str(
                bucket_lifecycle_rule)
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        else:
            log_exception = "Validate Bucket Lifecycle: " + str(
                bucket_name) + " and get the expected status code: " + str(
                bucket_lifecycle_rule)
            context.logger.debug(log_exception)
            TestLog.attach_allure_log(
                log_message=log_exception,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I validate bucket has no lifecycle")
def step_impl(context):
    """
    it should return error when bucket has no lifecycle configuration
    :param context:
    :return:
    """
    try:
        bucket_name = os.environ["bucket_name"]
        context.s3_resource.BucketLifecycle(bucket_name).rules
    except ClientError as ex:
        if 'The lifecycle configuration does not exist.' in str(ex):
            context.logger.debug("the bucket has no lifecycle configuration")


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete the lifecycle")
def step_impl(context):
    """
    :param context:
    :return:
    """
    try:
        bucket_name = os.environ["bucket_name"]
        response = context.s3_resource.BucketLifecycle(bucket_name).delete()
        context.logger.debug("delete the lifecycle: " + str(response))
        if response and str(response["ResponseMetadata"]["HTTPStatusCode"]) == "204":
            pass
        else:
            raise Exception("Delete bucket lifecycle configuration failed")
    except ClientError as ex:
        if 'The lifecycle configuration does not exist.' in str(ex):
            raise Exception("No lifecycle configuration found by the bucket")
        else:
            raise ex
