import os

from behave import *
from botocore.exceptions import ClientError

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog
from common_test_suite.templates.bucket_lifecycle import BucketLifecycleTemplate


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I Add Bucket Lifecycle Configuration {lifecycle_configuration_rule} to non-existing bucket")
def step_impl(context, lifecycle_configuration_rule):
    try:
        bucket_lifecycle_configuration_rule = BucketLifecycleTemplate.get_bucket_lifecycle_configuration_with_filter(
            **eval(lifecycle_configuration_rule))
        response = context.s3_resource.BucketLifecycleConfiguration("invalid-bucket").put(
            LifecycleConfiguration=bucket_lifecycle_configuration_rule)
        TestLog.attach_allure_log(
            log_message="Add Bucket Lifecycle configuration and Get the response: " + str(response),
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        context.error = str(ex)
        if "An error occurred (NoSuchBucket) when calling the PutBucketLifecycleConfiguration operation" in str(ex):
            TestLog.attach_allure_log(
                log_message="Put bucket lifecycle configuration and get the expected error message: " + str(ex),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        else:
            raise Exception("The expected exception is not returned as expected in the response: " + str(ex))


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I verify that the expected message {expected_message} return as expected")
def step_impl(context, expected_message):
    if context.error:
        actual_message = context.error
        if expected_message in str(actual_message):
            TestLog.attach_allure_log(
                log_message="Get the expected error message: " + str(actual_message),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        else:
            TestLog.attach_allure_log(
                log_message="The expected exception is not returned as expected: " + str(actual_message),
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
            raise Exception(
                "The expected exception is not returned as expected: " + str(actual_message))
    else:
        raise Exception


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I Add Bucket Lifecycle Configuration: {lifecycle_configuration_rule} with filter")
def step_impl(context, lifecycle_configuration_rule=None):
    bucket_name = DataGenerator.generate_bucket_name(context)
    logger = context.logger
    try:
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        bucket_lifecycle_configuration_rule = BucketLifecycleTemplate.get_bucket_lifecycle_configuration_with_filter(
            **eval(lifecycle_configuration_rule))
        context.logger.info(bucket_lifecycle_configuration_rule)
        http_status_code = context.s3_resource.BucketLifecycleConfiguration(bucket_name).put(
            LifecycleConfiguration=bucket_lifecycle_configuration_rule)
        context.actual_status_code_put_bucket_lifecycle = http_status_code
        put_bucket_lifecycle_configuration_rule = str(bucket_lifecycle_configuration_rule)
        log_message = "Add Bucket Lifecycle Configuration for bucket '" + bucket_name + "': " + put_bucket_lifecycle_configuration_rule + " and get the expected status code: " + str(
            http_status_code)
        logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        error_message = "Fail to add bucket lifecycle configuration for bucket '" + bucket_name + "', caused by the exception: " + str(
            ex)
        logger.error(error_message, exc_info=True)
        raise ex


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I Add Bucket Lifecycle Configuration: {lifecycle_rule} with prefix")
def step_impl(context, lifecycle_rule=None):
    bucket_name = DataGenerator.generate_bucket_name(context)
    logger = context.logger
    try:

        if context.boto3_client.s3_resource_check_bucket_exists(bucket_name):
            pass
        else:
            # create teh bucket if it doesn't exist
            context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        bucket_lifecycle_configuration_rule = BucketLifecycleTemplate.get_bucket_lifecycle_configuration_with_prefix(
            **eval(lifecycle_rule))
        http_status_code = context.s3_resource.BucketLifecycleConfiguration(bucket_name).put(
            LifecycleConfiguration=bucket_lifecycle_configuration_rule)
        context.actual_status_code_put_bucket_lifecycle = http_status_code
        context.put_bucket_lifecycle_configuration_rule = str(bucket_lifecycle_configuration_rule)
        log_message = "Add Bucket Lifecycle Configuration: " + str(
            bucket_name) + " and get the expected status code: " + str(
            lifecycle_rule)
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        error_message = "Fail to add bucket lifecycle configuration with prefix for bucket '" + bucket_name + "', caused by the exception: " + str(
            ex)
        logger.error(error_message, exc_info=True)
        raise ex


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I get Bucket lifecycle configuration")
def step_impl(context):
    bucket_name = os.environ["bucket_name"]
    bucket_lifecycle_configuration_rule = context.s3_resource.BucketLifecycleConfiguration(bucket_name).rules
    context.get_bucket_lifecycle_configuration_rule = bucket_lifecycle_configuration_rule
    log_message = "Get Bucket Lifecycle Configuration: " + str(
        bucket_name) + " and get the expected status code: " + str(
        bucket_lifecycle_configuration_rule)
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I validate Bucket lifecycle configuration")
def step_impl(context):
    bucket_name = os.environ["bucket_name"]
    bucket_lifecycle_configuration_rule = context.s3_resource.BucketLifecycleConfiguration(bucket_name).rules
    context.get_bucket_lifecycle_configuration_rule = bucket_lifecycle_configuration_rule
    if 'put_bucket_lifecycle_configuration_rule' in context and context.put_bucket_lifecycle_configuration_rule:
        if str(bucket_lifecycle_configuration_rule) in str(context.put_bucket_lifecycle_configuration_rule):
            log_message = "Validate Bucket Lifecycle: " + str(
                bucket_name) + " and get the expected bucket_lifecycle_configuration_rule: " + str(
                bucket_lifecycle_configuration_rule)
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        else:
            log_exception = "Validate Bucket Lifecycle: " + str(
                bucket_name) + " and get the unexpected bucket_lifecycle_configuration_rule: " + str(
                bucket_lifecycle_configuration_rule)
            context.logger.debug(log_exception)
            TestLog.attach_allure_log(
                log_message=log_exception,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I validate bucket has no lifecycle configuration")
def step_impl(context):
    """
    it should return error when bucket has no lifecycle configuration
    :param context:
    :return:
    """
    bucket_name = os.environ["bucket_name"]
    try:
        context.s3_resource.BucketLifecycleConfiguration(bucket_name).rules
    except ClientError as ex:
        if 'The lifecycle configuration does not exist.' in str(ex):
            log_message = "The bucket '" + bucket_name + "' has no lifecycle configuration"
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete the lifecycle configuration")
def step_impl(context):
    """
    :param context:
    :return:
    """
    try:
        bucket_name = os.environ["bucket_name"]
        response = context.s3_resource.BucketLifecycleConfiguration(bucket_name).delete()
        if response and str(response["ResponseMetadata"]["HTTPStatusCode"]) == "204":
            log_message = "Delete Bucket Lifecycle Configuration: " + str(
                bucket_name) + " and get the expected status code:204"
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        else:
            raise Exception("Delete bucket lifecycle configuration failed")
    except ClientError as ex:
        if 'The lifecycle configuration does not exist.' in str(ex):
            raise Exception("No lifecycle configuration found by the bucket")
