import json

from behave import *
from botocore.exceptions import ClientError

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog
from common_test_suite.templates.bucket_policy import BucketPolicyTemplate


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I Add bucket Policy to bucket with template: {bucket_policy_template}")
def step_impl(context, bucket_policy_template=None):
    """
    Add Bucket policy
    :param context:
    :param bucket_policy_template:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context)
    try:
        context.logger.debug("======> start to create the bucket " + bucket_name)
        http_status_code = context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
        context.logger.debug("======> end to create the bucket " + bucket_name)
        if bucket_policy_template:
            bucket_policy = BucketPolicyTemplate.get_bucket_policy(bucket_policy_template)
        else:
            raise Exception("Bucket Policy Template is not found as expected!")

        context.logger.debug("bucket policy template: " + str(bucket_policy))

        context.boto3_client.put_bucket_policy(bucket_name=bucket_name,
                                               bucket_policy=bucket_policy)
        context.actual_status_code_create_bucket = http_status_code
    except Exception as ex:
        raise Exception("Fail to add bucket policy " + str(ex))


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I validate bucket policy with contentMD5: {bucket_policy_template}")
def step_impl(context, bucket_policy_template=None):
    """
    Validate Bucket Policy
    :param context:
    :param bucket_policy_template:
    :return:
    """
    try:
        bucket_name = DataGenerator.generate_bucket_name(context)
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        if bucket_policy_template:
            bucket_policy = BucketPolicyTemplate.get_bucket_policy(bucket_policy_template)
        else:
            raise Exception("Bucket CORS Template is not found as expected!")
        policy_string = json.dumps(bucket_policy)
        context.s3_resource.BucketPolicy(bucket_name).put(Policy=policy_string, ContentMD5="12345654321")
        context.actual_status_code_create_bucket = 200
    except ClientError as ex:
        if 'The Content-MD5 you specified is not valid' in str(ex):
            context.logger.debug("wrong content-MD5 found")
        else:
            raise ex
