import json
import os

from behave import *
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I Put Bucket Replication with prefix {prefix} on ose endpoint {ose_endpoint_url} and tenant {tenant}")
def step_impl(context, prefix=None, tenant=None, ose_endpoint_url=None):
    try:
        vcd_site = os.environ["vcd_site"]
        source_bucket_name = os.environ["source_bucket_name"]
        destination_bucket_name = os.environ["destination_bucket_name"]
        x_vcloud_authorization = os.environ["Authorization"]
        req_header = context.prepare_request_header(x_vcloud_authorization=x_vcloud_authorization,
                                                    x_vmware_vcloud_tenant_context=os.environ[
                                                        vcd_site + "_" + tenant + "_X-VMWARE-VCLOUD-TENANT-CONTEXT"],
                                                    tenant=tenant)

        # Put Bucket Replication
        if prefix is None or prefix == 'None':
            req_payload = json.dumps({"rule": [
                {"id": "rule1", "status": "Enabled",
                 "destination": {"bucket": "arn:aws:s3:::" + destination_bucket_name + ""},
                 "prefix": ""}]})
        else:
            req_payload = json.dumps({"rule": [
                {"id": "rule1", "status": "Enabled",
                 "destination": {"bucket": "arn:aws:s3:::" + destination_bucket_name + ""},
                 "prefix": "" + prefix + ""}]})
        context.put_bucket_replication(
            req_url=os.environ[ose_endpoint_url] + "/" + source_bucket_name + "?replication",
            req_header=req_header,
            req_payload=req_payload)
    except Exception as ex:
        raise Exception("failed to send request!")
