import urllib

from behave import *

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I add Bucket tagging with parameters {parameters}")
def step_impl(context, parameters):
    """
    Upload Bucket with tag
    :param context:
    :param parameters:
    :return:
    """
    logger = context.logger
    # parse arguments
    tags = eval(parameters)
    bucket_name = DataGenerator.generate_bucket_name(context)
    try:

        context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
        bucket_tagging = context.s3_resource.BucketTagging(bucket_name)
        put_bucket_tagging_response = bucket_tagging.put(
            Tagging=tags
        )
        http_status_code = put_bucket_tagging_response["ResponseMetadata"]["HTTPStatusCode"]
        assert int(http_status_code) == 204
        log_message = "Put Bucket Tagging: " + str(
            str(put_bucket_tagging_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except Exception as ex:
        log_error = f"Fail to add Bucket tagging <" + bucket_name + "> \n and get the unexpected exception: " + str(
            ex)
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I get Bucket tagging")
def step_impl(context):
    """
    Get Bucket tag
    :param context:
    :return:
    """
    logger = context.logger
    bucket_name = context.bucket_name
    try:
        get_bkt_tagging_response = context.s3_client.get_bucket_tagging(
            Bucket=bucket_name
        )
        http_status_code = get_bkt_tagging_response["ResponseMetadata"]["HTTPStatusCode"]
        assert int(http_status_code) == 200
        if get_bkt_tagging_response:
            log_message = "Get Bucket Tagging: " + str(
                str(get_bkt_tagging_response))
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except Exception as ex:
        log_error = f"Fail to get Bucket tagging <" + bucket_name + "> \n and get the unexpected exception: " + str(
            ex)
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete Bucket tagging")
def step_impl(context):
    """
    Delete Bucket tag
    :param context:
    :param object_key:
    :return:
    """
    logger = context.logger
    bucket_name = context.bucket_name
    try:
        delete_bkt_tagging_response = context.s3_client.delete_bucket_tagging(
            Bucket=bucket_name
        )
        http_status_code = delete_bkt_tagging_response["ResponseMetadata"]["HTTPStatusCode"]
        assert int(http_status_code) == 204
        if delete_bkt_tagging_response:
            log_message = "Delete Bucket Tagging: " + str(
                str(delete_bkt_tagging_response))
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
    except Exception as ex:
        log_error = f"Fail to delete Bucket tagging <" + bucket_name + "> \n and get the unexpected exception: " + str(
            ex)
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)
