import urllib

from behave import *

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I add Bucket Website with parameters {parameters}")
def step_impl(context, parameters):
    """
    Add Bucket website
    :param context:
    :param parameters:
    :return:
    """
    # response = bucket_website.put(
    #     ChecksumAlgorithm='CRC32' | 'CRC32C' | 'SHA1' | 'SHA256',
    #     WebsiteConfiguration={
    #         'ErrorDocument': {
    #             'Key': 'string'
    #         },
    #         'IndexDocument': {
    #             'Suffix': 'string'
    #         },
    #         'RedirectAllRequestsTo': {
    #             'HostName': 'string',
    #             'Protocol': 'http' | 'https'
    #         },
    #         'RoutingRules': [
    #             {
    #                 'Condition': {
    #                     'HttpErrorCodeReturnedEquals': 'string',
    #                     'KeyPrefixEquals': 'string'
    #                 },
    #                 'Redirect': {
    #                     'HostName': 'string',
    #                     'HttpRedirectCode': 'string',
    #                     'Protocol': 'http' | 'https',
    #                     'ReplaceKeyPrefixWith': 'string',
    #                     'ReplaceKeyWith': 'string'
    #                 }
    #             },
    #         ]
    #     },
    #     ExpectedBucketOwner='string'
    # )
    logger = context.logger
    # parse arguments
    website_configuration = eval(parameters)
    bucket_name = DataGenerator.generate_bucket_name(context)
    try:

        context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
        bucket_website = context.s3_resource.BucketWebsite(bucket_name)
        put_bucket_website_response = bucket_website.put(
            WebsiteConfiguration=website_configuration
        )
        http_status_code = put_bucket_website_response["ResponseMetadata"]["HTTPStatusCode"]
        expected_status_code = "200"
        assert int(
            http_status_code) == int(
            expected_status_code), f"The excepted status code '" + expected_status_code + "'  but the actual status code '" + str(
            http_status_code) + "'"
        log_message = "Put Bucket Website: " + str(
            str(put_bucket_website_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except Exception as ex:
        log_error = f"Fail to add Bucket Website <" + bucket_name + "> \n and get the unexpected exception: " + str(ex)
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I get Bucket Website")
def step_impl(context):
    """
    Get Bucket Website
    :param context:
    :return:
    """
    logger = context.logger
    bucket_name = context.bucket_name
    try:
        get_bkt_website_response = context.s3_client.get_bucket_website(
            Bucket=bucket_name
        )
        http_status_code = get_bkt_website_response["ResponseMetadata"]["HTTPStatusCode"]
        expected_status_code = "200"
        assert int(
            http_status_code) == int(
            expected_status_code), f"The excepted status code '" + expected_status_code + "'  but the actual status code '" + str(
            http_status_code) + "'"
        if get_bkt_website_response:
            log_message = "Get Bucket Website: " + str(
                str(get_bkt_website_response))
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except Exception as ex:
        log_error = f"Fail to get Bucket Website <" + bucket_name + "> \n and get the unexpected exception: " + str(
            ex)
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete Bucket Website")
def step_impl(context):
    """
    Delete Bucket Website
    :param context:
    :return:
    """
    logger = context.logger
    bucket_name = context.bucket_name
    try:
        delete_bkt_website_response = context.s3_client.delete_bucket_website(
            Bucket=bucket_name
        )
        http_status_code = delete_bkt_website_response["ResponseMetadata"]["HTTPStatusCode"]
        expected_status_code = "204"
        assert int(
            http_status_code) == int(
            expected_status_code), f"The excepted status code '" + expected_status_code + "'  but the actual status code '" + str(
            http_status_code) + "'"
        if delete_bkt_website_response:
            log_message = "Delete Bucket Website: " + str(
                str(delete_bkt_website_response))
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
    except Exception as ex:
        log_error = f"Fail to delete Bucket Website <" + bucket_name + "> \n and get the unexpected exception: " + str(
            ex)
        logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)
