import datetime

from behave import *
from botocore.exceptions import ClientError

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I copy object to key: {object_key}")
def step_impl(context, object_key):
    """
    Copy object
    :param context:
    :param object_key:
    :return:
    """
    try:
        src_bucket_name = context.bucket_name
        context.src_bucket_name = src_bucket_name
        copy_source = {
            "Bucket": src_bucket_name,
            "Key": object_key
        }
        # prepare the destination bucket
        dest_bucket_name = "dest-api-test-" + DataGenerator().id_generator_lowercase(size=15)
        context.dest_bucket_name = dest_bucket_name
        create_destination_bucket_response = context.s3_resource.Bucket(dest_bucket_name).create()
        if create_destination_bucket_response:
            pass
        else:
            log_exception = "Fail to create the destination bucket '" + dest_bucket_name + "' and the get the exception: " + str(
                create_destination_bucket_response)
            TestLog.attach_allure_log(
                log_message=log_exception,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
            raise Exception(log_exception)
        # copy object from source bucket
        dest = context.s3_resource.Bucket(dest_bucket_name)
        copy_object_response = dest.copy(copy_source, object_key)
        log_message = "I copy object to key [" + object_key + "] from source bucket:" + str(
            src_bucket_name) + " and get the response: " + str(
            str(copy_object_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        context.logger.error(ex.response)
        raise Exception(ex.response)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I copy object from source bucket with key {object_key} and parameters {copy_parameters}")
def step_impl(context, object_key, copy_parameters):
    """
    Copy object with parameters
    :param context:
    :param object_key:
    :param copy_parameters:
    :return:
    """
    # response = object.copy_from(
    #     ACL='private' | 'public-read' | 'public-read-write' | 'authenticated-read' | 'aws-exec-read' | 'bucket-owner-read' | 'bucket-owner-full-control',
    #     CacheControl='string',
    #     ChecksumAlgorithm='CRC32' | 'CRC32C' | 'SHA1' | 'SHA256',
    #     ContentDisposition='string',
    #     ContentEncoding='string',
    #     ContentLanguage='string',
    #     ContentType='string',
    #     CopySource='string' or {'Bucket': 'string', 'Key': 'string', 'VersionId': 'string'},
    #     CopySourceIfMatch='string',
    #     CopySourceIfModifiedSince=datetime(2015, 1, 1),
    #     CopySourceIfNoneMatch='string',
    #     CopySourceIfUnmodifiedSince=datetime(2015, 1, 1),
    #     Expires=datetime(2015, 1, 1),
    #     GrantFullControl='string',
    #     GrantRead='string',
    #     GrantReadACP='string',
    #     GrantWriteACP='string',
    #     Metadata={
    #         'string': 'string'
    #     },
    #     MetadataDirective='COPY' | 'REPLACE',
    #     TaggingDirective='COPY' | 'REPLACE',
    #     ServerSideEncryption='AES256' | 'aws:kms',
    #     StorageClass='STANDARD' | 'REDUCED_REDUNDANCY' | 'STANDARD_IA' | 'ONEZONE_IA' | 'INTELLIGENT_TIERING' | 'GLACIER' | 'DEEP_ARCHIVE' | 'OUTPOSTS' | 'GLACIER_IR',
    #     WebsiteRedirectLocation='string',
    #     SSECustomerAlgorithm='string',
    #     SSECustomerKey='string',
    #     SSEKMSKeyId='string',
    #     SSEKMSEncryptionContext='string',
    #     BucketKeyEnabled=True | False,
    #     CopySourceSSECustomerAlgorithm='string',
    #     CopySourceSSECustomerKey='string',
    #     RequestPayer='requester',
    #     Tagging='string',
    #     ObjectLockMode='GOVERNANCE' | 'COMPLIANCE',
    #     ObjectLockRetainUntilDate=datetime(2015, 1, 1),
    #     ObjectLockLegalHoldStatus='ON' | 'OFF',
    #     ExpectedBucketOwner='string',
    #     ExpectedSourceBucketOwner='string'
    # )
    source_bucket_name = context.bucket_name
    context.src_bucket_name = source_bucket_name
    source_obj = context.s3_resource.Object(source_bucket_name, object_key)
    copy_source = {
        "Bucket": source_bucket_name,
        "Key": object_key
    }
    # parse arguments
    kwargs = eval(copy_parameters)

    # prepare the destination bucket
    dest_bucket_name = "dest-api-test-" + DataGenerator().id_generator_lowercase(size=15)
    context.dest_bucket_name = dest_bucket_name
    create_destination_bucket_response = context.s3_resource.Bucket(dest_bucket_name).create()
    if create_destination_bucket_response:
        pass
    else:
        log_exception = "Fail to create the destination bucket '" + dest_bucket_name + "' and the get the exception: " + str(
            create_destination_bucket_response)
        TestLog.attach_allure_log(
            log_message=log_exception,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_exception)
    # copy object from source bucket
    kwargs['CopySource'] = copy_source
    # e-tag
    for key, value in kwargs.items():
        if "{{" in value:
            context.logger.debug("object key: " + source_obj.key)
            replaced_value = value.replace(value, source_obj.e_tag)
            kwargs[key] = replaced_value
        if key == "CopySourceIfModifiedSince" or key == "CopySourceIfUnmodifiedSince":
            test_time = eval(value)
            kwargs[key] = test_time

    dest_obj = context.s3_resource.Object(dest_bucket_name, object_key)
    copy_object_response = dest_obj.copy_from(**kwargs)
    log_message = "I copy object to key [" + object_key + "] from source bucket:" + str(
        source_bucket_name) + " and get the response: " + str(
        str(copy_object_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then(
    "I should fail to copy object from source bucket with key {object_key} and parameters {copy_parameters} when non matched found")
def step_impl(context, object_key, copy_parameters):
    source_bucket_name = context.bucket_name
    context.src_bucket_name = source_bucket_name
    source_obj = context.s3_resource.Object(source_bucket_name, object_key)
    copy_source = {
        "Bucket": source_bucket_name,
        "Key": object_key
    }
    # parse arguments
    kwargs = eval(copy_parameters)

    # prepare the destination bucket
    dest_bucket_name = "dest-api-test-" + DataGenerator().id_generator_lowercase(size=15)
    context.dest_bucket_name = dest_bucket_name
    create_destination_bucket_response = context.s3_resource.Bucket(dest_bucket_name).create()
    if create_destination_bucket_response:
        pass
    else:
        log_exception = "Fail to create the destination bucket '" + dest_bucket_name + "' and the get the exception: " + str(
            create_destination_bucket_response)
        TestLog.attach_allure_log(
            log_message=log_exception,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_exception)
    # copy object from source bucket
    kwargs['CopySource'] = copy_source
    # e-tag
    for key, value in kwargs.items():
        if "{{" in value:
            replaced_value = value.replace(value, source_obj.e_tag)
            kwargs[key] = replaced_value
        if key == "CopySourceIfModifiedSince" or key == "CopySourceIfUnmodifiedSince":
            test_time = eval(value)
            context.logger.debug("test time: " + str(test_time))
            kwargs[key] = test_time
    dest_obj = context.s3_resource.Object(dest_bucket_name, object_key)
    try:
        copy_object_response = dest_obj.copy_from(**kwargs)
        log_message = "Copy object and get the response: " + str(
            str(copy_object_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        context.error = str(ex)
        log_message = "Copy object and get the exception: " + str(
            str(ex))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then(
    "I should fail to copy object from source bucket with key {object_key} and parameters {copy_parameters} when matched pre-condition found")
def step_impl(context, object_key, copy_parameters):
    source_bucket_name = context.bucket_name
    context.src_bucket_name = source_bucket_name
    source_obj = context.s3_resource.Object(source_bucket_name, object_key)
    copy_source = {
        "Bucket": source_bucket_name,
        "Key": object_key
    }
    # parse arguments
    kwargs = eval(copy_parameters)

    # prepare the destination bucket
    dest_bucket_name = "dest-api-test-" + DataGenerator().id_generator_lowercase(size=15)
    context.dest_bucket_name = dest_bucket_name
    create_destination_bucket_response = context.s3_resource.Bucket(dest_bucket_name).create()
    if create_destination_bucket_response:
        pass
    else:
        log_exception = "Fail to create the destination bucket '" + dest_bucket_name + "' and the get the exception: " + str(
            create_destination_bucket_response)
        TestLog.attach_allure_log(
            log_message=log_exception,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_exception)
    # copy object from source bucket
    kwargs['CopySource'] = copy_source
    # e-tag
    for key, value in kwargs.items():
        if "{{" in value:
            context.logger.debug("object key: " + source_obj.key)
            replaced_value = value.replace(value, source_obj.e_tag)
            kwargs[key] = replaced_value
    dest_obj = context.s3_resource.Object(dest_bucket_name, object_key)
    try:
        copy_object_response = dest_obj.copy_from(**kwargs)
        log_message = "Copy object and get the response: " + str(
            str(copy_object_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        context.error = str(ex)
        log_message = "Copy object and get the exception: " + str(
            str(ex))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete the source bucket with object key {object_key}")
def step_impl(context, object_key):
    src_bucket_name = context.src_bucket_name
    context.logger.debug("======> start to delete objects from the bucket")
    response = context.boto3_client.s3_delete_all_objects(bucket_name=src_bucket_name)
    if response:
        if isinstance(response, bool):
            pass
        else:
            context.logger.debug("delete object's status: " + str(response['ResponseMetadata']['HTTPStatusCode']))
    else:
        s3_object = context.s3_resource.Object(src_bucket_name, object_key)
        bucket = context.s3_resource.Bucket(src_bucket_name)
        try:
            objs = list(bucket.objects.filter(Prefix=object_key))
            if len(objs) > 0:
                context.logger.debug("======> start to delete objects from the bucket")
                response = s3_object.delete()
                context.logger.debug("======> end to delete objects from the bucket" + str(response))
                context.logger.debug("delete object's status: " + str(response['ResponseMetadata']['HTTPStatusCode']))
                context.logger.debug("======> end to delete objects from the bucket" + src_bucket_name)
            else:
                context.logger.debug("key doesn't exist!")
        except ClientError as ce:
            TestLog.attach_allure_log(
                log_message="Fail to delete object(s): " + str(ce),
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)

    context.logger.debug("======> end to delete objects from the bucket" + src_bucket_name)
    # delete bucket
    http_status_code = context.boto3_client.s3_delete_bucket(bucket_name=src_bucket_name)
    context.actual_status_code_delete_bucket = http_status_code
    log_message = "Source Bucket deleted successfully: " + src_bucket_name
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete the destination bucket with object key {object_key}")
def step_impl(context, object_key):
    """
    Delete source/destination bucket
    :param context:
    :param object_key:
    :return:
    """
    dest_bucket_name = context.dest_bucket_name
    response = context.boto3_client.s3_delete_all_objects(bucket_name=dest_bucket_name)
    if response:
        if isinstance(response, bool):
            pass
        else:
            context.logger.debug("delete object's status: " + str(response['ResponseMetadata']['HTTPStatusCode']))
    else:
        s3_object = context.s3_resource.Object(dest_bucket_name, object_key)
        context.logger.debug("======> start to delete objects from the bucket")
        bucket = context.s3_resource.Bucket(dest_bucket_name)
        try:
            objs = list(bucket.objects.filter(Prefix=object_key))
            if len(objs) > 0:
                context.logger.debug("======> start to delete objects from the bucket")
                response = s3_object.delete()
                context.logger.debug("======> end to delete objects from the bucket" + str(response))
                context.logger.debug("delete object's status: " + str(response['ResponseMetadata']['HTTPStatusCode']))
                context.logger.debug("======> end to delete objects from the bucket" + dest_bucket_name)
            else:
                context.logger.debug("key doesn't exist!")
        except ClientError as ce:
            TestLog.attach_allure_log(
                log_message="Fail to delete object(s): " + str(ce),
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        context.logger.debug("======> end to delete objects from the bucket" + dest_bucket_name)
    # delete bucket
    http_status_code = context.boto3_client.s3_delete_bucket(bucket_name=dest_bucket_name)
    context.actual_status_code_delete_bucket = http_status_code
    log_message = "Destination Bucket deleted successfully: " + dest_bucket_name
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
