import datetime
import re

from behave import *
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should HEAD object with key {object_key} and parameters {head_object_parameters} successfully")
def step_impl(context, object_key, head_object_parameters):
    """
    Head object with parameters
    :param context:
    :param object_key:
    :param head_object_parameters:
    :return:
    """
    # head_object(
    #     Bucket='string',
    #     IfMatch='string',
    #     IfModifiedSince=datetime(2015, 1, 1),
    #     IfNoneMatch='string',
    #     IfUnmodifiedSince=datetime(2015, 1, 1),
    #     Key='string',
    #     Range='string',
    #     VersionId='string',
    #     SSECustomerAlgorithm='string',
    #     SSECustomerKey='string',
    #     RequestPayer='requester',
    #     PartNumber=123,
    #     ExpectedBucketOwner='string',
    #     ChecksumMode='ENABLED'
    # )
    bucket_name = context.bucket_name
    # parse arguments
    kwargs = eval(head_object_parameters)
    kwargs["Bucket"] = bucket_name
    kwargs["Key"] = object_key
    for key, value in kwargs.items():
        if "{{" in str(value):
            # handle e_tag
            obj = context.s3_resource.Object(bucket_name, object_key)
            variables_to_replace = re.findall("{{.*?}}", str(value))
            for var in variables_to_replace:
                variable = str(var).replace("{{", "").replace("}}", "")
                if "e_tag" == variable:
                    replaced_value = str(value).replace(var, obj.e_tag)
                if "version_id" == variable:
                    # List version of objects filtered by the key
                    s3_bucket = context.s3_resource.Bucket(bucket_name)
                    obj_versions = s3_bucket.object_versions.all()
                    for obj_version in obj_versions:
                        object_version_id = str(obj_version.id)
                    replaced_value = str(value).replace(var, object_version_id)
            kwargs[key] = replaced_value
        if key == "IfModifiedSince" or key == "IfUnmodifiedSince":
            test_time = eval(value)
            kwargs[key] = test_time
    # head object
    head_object_response = context.s3_client.head_object(**kwargs)
    if head_object_response:
        log_message = "Head Object successfully for the bucket '" + bucket_name + "'"
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    else:
        log_exception = "Head Object failed for the bucket '" + bucket_name + "' and get the response: " + str(
            head_object_response)
        context.logger.error(log_exception)
        TestLog.attach_allure_log(
            log_message=log_exception,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_exception)
