import datetime
import re

from behave import *
from botocore.exceptions import ClientError

from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should List objects with parameters {list_object_parameters} successfully")
def step_impl(context, list_object_parameters):
    """
    List object with parameters
    :param context:
    :param list_object_parameters:
    :return:
    """
    # list_objects_v2(
    #     Bucket='string',
    #     Delimiter='string',
    #     EncodingType='url',
    #     MaxKeys=123,
    #     Prefix='string',
    #     ContinuationToken='string',
    #     FetchOwner=True | False,
    #     StartAfter='string',
    #     RequestPayer='requester',
    #     ExpectedBucketOwner='string'
    # )
    bucket_name = context.bucket_name
    # parse arguments
    kwargs = eval(list_object_parameters)
    kwargs["Bucket"] = bucket_name
    for key, value in kwargs.items():
        if "{{" in str(value):
            # handle version id
            # List version of objects filtered by the key
            s3_bucket = context.s3_resource.Bucket(bucket_name)
            obj_versions = s3_bucket.object_versions.all()
            for obj_version in obj_versions:
                object_version_id = str(obj_version.id)
            variables_to_replace = re.findall("{{.*?}}", str(value))
            for var in variables_to_replace:
                variable = str(var).replace("{{", "").replace("}}", "")
                if "version_id" == variable:
                    replaced_value = str(value).replace(var, object_version_id)
            kwargs[key] = replaced_value
        if key == "IfModifiedSince" or key == "IfUnmodifiedSince":
            test_time = eval(value)
            kwargs[key] = test_time
    # if list_objects_v2_response:
    context.logger.debug("====== list_objects_v2_response ======")
    list_objects_response = context.s3_client.list_objects(**kwargs)
    log_message = "List objects and get the response: " + str(
        str(list_objects_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    context.logger.debug("====== list_objects_v2_response ======")


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should fail to list objects with parameters {list_object_parameters} with negative value")
def step_impl(context, list_object_parameters):
    """
    Negative test, list object with negative parameters
    :param context:
    :param list_object_parameters:
    :return:
    """
    bucket_name = context.bucket_name
    # parse arguments
    kwargs = eval(list_object_parameters)
    kwargs["Bucket"] = bucket_name
    for key, value in kwargs.items():
        # if "{{" in str(value):
        #     # handle e_tag
        #     obj = context.s3_resource.Object(bucket_name, object_key)
        #     variables_to_replace = re.findall("{{.*?}}", str(value))
        #     for var in variables_to_replace:
        #         variable = str(var).replace("{{", "").replace("}}", "")
        #         if "e_tag" == variable:
        #             replaced_value = str(value).replace(var, obj.e_tag)
        #     kwargs[key] = replaced_value
        if key == "IfModifiedSince" or key == "IfUnmodifiedSince":
            test_time = eval(value)
            kwargs[key] = test_time

    # if list_objects_v2_response:
    try:
        context.logger.debug("====== list_objects_v2_response ======")
        list_objects_response = context.s3_client.list_objects(**kwargs)
        log_message = "List objects and get the response: " + str(
            str(list_objects_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        context.logger.debug("====== list_objects_v2_response ======")
    except ClientError as ex:
        context.error = str(ex)
        log_message = "List objects and get the exception: " + str(
            str(ex))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
