import datetime
import re

from behave import *
from botocore.exceptions import ClientError

from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should List object V2 with parameters {list_object_v2_parameters} successfully")
def step_impl(context, list_object_v2_parameters):
    """
    List Object V2 with parameters
    :param context:
    :param list_object_v2_parameters:
    :return:
    """
    # list_objects_v2(
    #     Bucket='string',
    #     Delimiter='string',
    #     EncodingType='url',
    #     MaxKeys=123,
    #     Prefix='string',
    #     ContinuationToken='string',
    #     FetchOwner=True | False,
    #     StartAfter='string',
    #     RequestPayer='requester',
    #     ExpectedBucketOwner='string'
    # )
    try:
        bucket_name = context.bucket_name
        # parse arguments
        kwargs = eval(list_object_v2_parameters)
        kwargs["Bucket"] = bucket_name
        for key, value in kwargs.items():
            if key == "IfModifiedSince" or key == "IfUnmodifiedSince":
                test_time = eval(value)
                context.logger.debug("test time: " + str(test_time))
                kwargs[key] = test_time
        # list_objects_v2_response
        context.logger.debug("====== list_objects_v2_response ======")
        list_objects_v2_response = context.s3_client.list_objects_v2(**kwargs)
        if list_objects_v2_response:
            log_message = "List object v2 successfully and get the response: " + str(list_objects_v2_response)
            TestLog.attach_allure_log(
                log_message=str(log_message),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        log_message = "List object v2 failed and get the exception: " + str(ex)
        context.logger.error(log_message)
        TestLog.attach_allure_log(
            log_message=str(log_message),
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_message)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should fail to list object V2 with parameters {list_object_v2_parameters} with negative value")
def step_impl(context, list_object_v2_parameters):
    bucket_name = context.bucket_name
    # parse arguments
    kwargs = eval(list_object_v2_parameters)
    kwargs["Bucket"] = bucket_name
    for key, value in kwargs.items():
        if key == "IfModifiedSince" or key == "IfUnmodifiedSince":
            test_time = eval(value)
            kwargs[key] = test_time
    # list_objects_v2_response:
    try:
        context.logger.debug("====== list_objects_v2 ======")
        list_objects_v2_response = context.s3_client.list_objects_v2(**kwargs)
        log_message = "List objects and get the response: " + str(
            str(list_objects_v2_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        context.logger.debug("====== list_objects_v2 ======")
    except ClientError as ex:
        context.error = str(ex)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I verify that expected error {expected_error} is found")
def step_impl(context, expected_error):
    actual_error = context.error
    if expected_error in actual_error:
        log_message = "Expected error is found: " + str(expected_error)
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    else:
        log_exception = "Expected error '" + str(
            expected_error) + "' is NOT found as expected, and the actual value is '" + actual_error + "'"
        TestLog.attach_allure_log(
            log_message=log_exception,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_exception)
