import datetime
import re

from behave import *
from botocore.exceptions import ClientError

from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should List object versions with parameters {list_object_versions_parameters} successfully")
def step_impl(context, list_object_versions_parameters):
    # list_object_versions(
    #     Bucket='string',
    #     Delimiter='string',
    #     EncodingType='url',
    #     KeyMarker='string',
    #     MaxKeys=123,
    #     Prefix='string',
    #     VersionIdMarker='string',
    #     ExpectedBucketOwner='string'
    # )
    try:
        bucket_name = context.bucket_name
        # parse arguments
        kwargs = eval(list_object_versions_parameters)
        kwargs["Bucket"] = bucket_name
        for key, value in kwargs.items():
            if "{{" in str(value):
                # List version of objects filtered by the key
                s3_bucket = context.s3_resource.Bucket(bucket_name)
                obj_versions = s3_bucket.object_versions.all()
                for obj_version in obj_versions:
                    object_version_id = str(obj_version.id)
                    if object_version_id:
                        break
                variables_to_replace = re.findall("{{.*?}}", str(value))
                for var in variables_to_replace:
                    variable = str(var).replace("{{", "").replace("}}", "")
                    if "version_id" == variable:
                        replaced_value = str(value).replace(var, object_version_id)
                        kwargs[key] = replaced_value
            if key == "IfModifiedSince" or key == "IfUnmodifiedSince":
                test_time = eval(value)
                kwargs[key] = test_time
        # list object versions
        context.logger.debug("====== list_object_versions_response ======")
        TestLog.attach_allure_log(
            log_message=str(kwargs),
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        list_object_version_response = context.s3_client.list_object_versions(**kwargs)
        log_message = "List objects and get the response: " + str(
            str(list_object_version_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        # context.boto3_client.pretty_context.logger.debug(list_object_version_response)
        context.logger.debug("====== list_object_versions_response ======")
    except ClientError as ex:
        log_error = "Call method 'list_object_versions()' failed and get unexpected exception: " + str(ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(ex.response)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should fail to list object versions with parameters {list_object_versions_parameters} with negative value")
def step_impl(context, list_object_versions_parameters):
    bucket_name = context.bucket_name
    # parse arguments
    kwargs = eval(list_object_versions_parameters)
    kwargs["Bucket"] = bucket_name
    for key, value in kwargs.items():
        if "{{" in str(value):
            # handle version id
            # List version of objects filtered by the key
            s3_bucket = context.s3_resource.Bucket(bucket_name)
            obj_versions = s3_bucket.object_versions.all()
            for obj_version in obj_versions:
                object_version_id = str(obj_version.id)
                context.logger.debug(type(object_version_id))
                context.logger.debug("object version id: " + object_version_id)

            variables_to_replace = re.findall("{{.*?}}", str(value))
            for var in variables_to_replace:
                variable = str(var).replace("{{", "").replace("}}", "")
                if "version_id" == variable:
                    replaced_value = str(value).replace(var, object_version_id)
            kwargs[key] = replaced_value
        if key == "IfModifiedSince" or key == "IfUnmodifiedSince":
            test_time = eval(value)
            context.logger.debug("test time: " + str(test_time))
            kwargs[key] = test_time
    # list object versions
    try:
        context.logger.debug("====== list_object_versions_response ======")
        list_object_version_response = context.s3_client.list_object_versions(**kwargs)
        log_message = "List objects and get the response: " + str(
            str(list_object_version_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        context.boto3_client.pretty_context.logger.debug(list_object_version_response)
        context.logger.debug("====== list_object_versions_response ======")
    except ClientError as ex:
        context.error = str(ex)
        log_message = "List object versions and get the expected exception: " + str(
            str(ex))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
