import datetime
import os
import re
from behave import *
from botocore.exceptions import ClientError

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


def _add_header(request):
    request.headers.add_header("x-amz-grant-full-control", id="170f126d-c378-45f9-9278-2c4caaabd294")


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I should create multipart upload successfully with {create_multipart_upload_parameters}")
def step_impl(context, create_multipart_upload_parameters):
    kwargs = {}
    try:
        if context.bucket_name:
            bucket_name = context.bucket_name
        else:
            bucket_name = DataGenerator().id_generator_lowercase(size=15)
            os.environ["bucket_name"] = bucket_name
            context.bucket_name = bucket_name
        # parse arguments
        kwargs = eval(create_multipart_upload_parameters)
        # create_multipart_upload
        context.logger.debug("====== initiate_multipart_upload ======")
        http_status_code = context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name,
                                                                          object_lock_enabled_for_bucket=True)
        context.logger.debug("create bucket and get the http status code: " + str(http_status_code))
        TestLog.attach_allure_log(
            log_message="Create bucket and get the http status code: " + str(http_status_code),
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        canonical_id = ''
        if "Grant" in str(kwargs):
            if "{{" in str(kwargs):
                source_string = str(create_multipart_upload_parameters)
                variables_to_replace = re.findall("{{.*?}}", source_string)
                for variable in variables_to_replace:
                    key = str(variable).replace("{{", "").replace("}}", "")
                    canonical_id = eval(os.environ[key])['canonical_id']
                if canonical_id:
                    kwargs = eval(source_string.replace(variables_to_replace[0], canonical_id))
        # special handle datetime, e.g. ObjectLockRetainUntilDate
        for key, value in kwargs.items():
            if "ObjectLockRetainUntilDate" in str(key) or "Expires" in str(key):
                test_time = eval(value)
                kwargs[key] = test_time
                break
        kwargs["Bucket"] = bucket_name
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            bucket = context.s3_resource.Bucket(bucket_name)
            object_key = kwargs["Key"]
            initiate_multipart_upload_response = bucket.Object(object_key).initiate_multipart_upload(**kwargs)
            context.mpu = initiate_multipart_upload_response
            context.upload_id = str(initiate_multipart_upload_response.id)
            part = context.mpu.Part(1)
            part_info = {
                'Parts': []
            }
            # upload file, the minimum size is 5Mib
            if "/" in object_key:
                index = object_key.rfind("/")
                file_path = object_key[index + 1:]
                response = part.upload(Body=open(os.path.join(os.environ["test_templates_directory"], file_path), 'rb'))
            else:
                response = part.upload(Body=open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb'))
            part_info['Parts'].append({
                'PartNumber': 1,
                'ETag': response['ETag']
            })
            context.logger.info(
                "initiate multipart upload for bucket '" + bucket_name + "': and get the response " + str(
                    initiate_multipart_upload_response))
            log_message = "create_multipart_upload: " + str(
                str(initiate_multipart_upload_response))
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            context.upload_id = str(initiate_multipart_upload_response.id)
            context.mpu = initiate_multipart_upload_response
            context.logger.debug("====== initiate_multipart_upload ======")
    except ClientError as ex:
        log_error = f"Call method 'initiate_multipart_upload('" + str(
            kwargs) + "')' failed \n and get unexpected exception: " + str(
            ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should abort multipart upload successfully with {abort_multipart_upload_parameters}")
def step_impl(context, abort_multipart_upload_parameters):
    # Request Syntax
    # response = client.abort_multipart_upload(
    #     Bucket='string',
    #     Key='string',
    #     UploadId='string',
    #     RequestPayer='requester',
    #     ExpectedBucketOwner='string'
    # )

    bucket_name = context.bucket_name
    upload_id = context.upload_id
    # parse arguments
    kwargs = eval(abort_multipart_upload_parameters)
    kwargs["Bucket"] = bucket_name
    kwargs["UploadId"] = upload_id
    # create_multipart_upload
    context.logger.debug("====== abort_multipart_upload ======")
    abort_multipart_upload_response = context.s3_client.abort_multipart_upload(**kwargs)
    log_message = "abort_multipart_upload: " + str(
        str(abort_multipart_upload_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    context.logger.debug("====== abort_multipart_upload ======")


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should complete multipart upload successfully with {complete_multipart_upload_parameters}")
def step_impl(context, complete_multipart_upload_parameters):
    """

    :param context:
    :param complete_multipart_upload_parameters:
    :return:
    """
    try:
        bucket_name = context.bucket_name
        object_key = eval(complete_multipart_upload_parameters)["Key"]
        bucket = context.s3_resource.Bucket(bucket_name)
        part = context.mpu.Part(1)
        part_info = {
            'Parts': []
        }
        # upload file, the minimum size is 5Mib
        if "/" in object_key:
            index = object_key.rfind("/")
            file_path = object_key[index + 1:]
            response = part.upload(Body=open(os.path.join(os.environ["test_templates_directory"], file_path), 'rb'))
        else:
            response = part.upload(Body=open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb'))
        part_info['Parts'].append({
            'PartNumber': 1,
            'ETag': response['ETag']
        })
        complete_multipart_upload_response = context.mpu.complete(MultipartUpload=part_info)
        bucket.Object(object_key).get()
        log_message = "complete_multipart_upload_response: " + str(
            str(complete_multipart_upload_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        log_error = f"Call method 'complete_multipart_upload('" + str(
            complete_multipart_upload_parameters) + "')' failed \n and get unexpected exception: " + str(
            ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should be able to upload_part successfully with {upload_part_parameters}")
def step_impl(context, upload_part_parameters):
    """

    :param context:
    :param upload_part_parameters:
    :return:
    """
    # Request Syntax
    #
    # response = client.upload_part(
    #     Body=b'bytes' | file,
    #     Bucket='string',
    #     ContentLength=123,
    #     ContentMD5='string',
    #     ChecksumAlgorithm='CRC32' | 'CRC32C' | 'SHA1' | 'SHA256',
    #     ChecksumCRC32='string',
    #     ChecksumCRC32C='string',
    #     ChecksumSHA1='string',
    #     ChecksumSHA256='string',
    #     Key='string',
    #     PartNumber=123,
    #     UploadId='string',
    #     SSECustomerAlgorithm='string',
    #     SSECustomerKey='string',
    #     RequestPayer='requester',
    #     ExpectedBucketOwner='string'
    # )

    bucket_name = context.bucket_name
    upload_id = context.upload_id
    # parse arguments
    object_key = eval(upload_part_parameters)["Key"]
    kwargs = eval(upload_part_parameters)
    kwargs["Bucket"] = bucket_name
    kwargs["UploadId"] = upload_id
    # upload file, the minimum size is 5Mib
    if "/" in object_key:
        index = object_key.rfind("/")
        file_path = object_key[index + 1:]
        kwargs["Body"] = open(os.path.join(os.environ["test_templates_directory"], file_path), 'rb')
    else:
        kwargs["Body"] = open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb')

    # complete_multipart_upload
    context.logger.debug("====== upload_part ======")
    upload_part_response = context.s3_client.upload_part(**kwargs)
    log_message = "upload_part: " + str(
        str(upload_part_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
    context.logger.debug("====== upload_part ======")


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should be able to list multipart upload successfully with {list_multipart_uploads_parameters}")
def step_impl(context, list_multipart_uploads_parameters):
    """

    :param context:
    :param list_multipart_uploads_parameters:
    :return:
    """
    # Request Syntax
    # response = client.list_multipart_uploads(
    #     Bucket='string',
    #     Delimiter='string',
    #     EncodingType='url',
    #     KeyMarker='string',
    #     MaxUploads=123,
    #     Prefix='string',
    #     UploadIdMarker='string',
    #     ExpectedBucketOwner='string'
    # )

    bucket_name = context.bucket_name
    # parse arguments
    kwargs = eval(list_multipart_uploads_parameters)
    kwargs["Bucket"] = bucket_name
    for key, value in kwargs.items():
        if "UploadIdMarker" in str(key):
            source_string = str(kwargs)
            variables_to_replace = re.findall("{{.*?}}", source_string)
            value = context.upload_id
            kwargs = eval(source_string.replace(variables_to_replace[0], value))

    # list_multipart_uploads
    context.logger.debug("====== list_multipart_uploads ======")
    abort_multipart_upload_response = context.s3_client.list_multipart_uploads(**kwargs)
    log_message = "list_multipart_uploads: " + str(
        str(abort_multipart_upload_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
    context.logger.debug("====== list_multipart_uploads ======")
