import os
import re
import datetime

from behave import *
from botocore.exceptions import ClientError

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I put object {other_object_key} to bucket")
def step_impl(context, other_object_key):
    """
    Upload file object with key
    :param context:
    :param other_object_key:
    :return:
    """
    bucket_name = context.bucket_name
    context.logger.debug("======> start to upload objects <" + other_object_key + "> to the bucket " + bucket_name)
    if "#" in other_object_key:
        split_object_key = other_object_key.split("#")
        for object_key in split_object_key:
            context.logger.debug("======> start to upload objects <" + object_key + "> to the bucket " + bucket_name)
            upload_object_response = context.s3_resource.Object(bucket_name, object_key).put(
                Body=open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb'))
            if context.boto3_client.s3_waiters_check_object_exists(bucket_name=bucket_name, key_name=object_key):
                log_message = "Put Object and get the response: " + str(
                    str(upload_object_response))
                context.logger.debug(log_message)
                TestLog.attach_allure_log(
                    log_message=log_message,
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
                assert upload_object_response['ResponseMetadata']['HTTPStatusCode'] == 200
            context.logger.debug("======> start to upload objects <" + object_key + "> to the bucket " + bucket_name)

    else:
        context.logger.debug("======> start to upload objects <" + other_object_key + "> to the bucket " + bucket_name)
        upload_object_response = context.s3_resource.Object(bucket_name, other_object_key).put(
            Body=open(os.path.join(os.environ["test_templates_directory"], other_object_key), 'rb'))
        if context.boto3_client.s3_waiters_check_object_exists(bucket_name=bucket_name, key_name=other_object_key):
            log_message = "Put Object and get the response: " + str(
                str(upload_object_response))
            context.logger.debug(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            assert upload_object_response['ResponseMetadata']['HTTPStatusCode'] == 200
        context.logger.debug("======> end to upload objects <" + other_object_key + "> to the bucket " + bucket_name)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I put Path object {path_object_key} to bucket")
def step_impl(context, path_object_key):
    """
    Upload folder object with key
    :param context:
    :param path_object_key:
    :return:
    """
    bucket_name = context.bucket_name
    upload_object_response = context.s3_resource.Object(bucket_name, path_object_key).put(Body="test")
    if context.boto3_client.s3_waiters_check_object_exists(bucket_name=bucket_name, key_name=path_object_key):
        log_message = "Put Object and get the response: " + str(
            str(upload_object_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        assert upload_object_response['ResponseMetadata']['HTTPStatusCode'] == 200


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I upload object to bucket with key {object_key} and parameters {parameters}")
def step_impl(context, object_key, parameters):
    # response = object.put(
    #     ACL='' | 'public-read' | 'public-read-write' | 'authenticated-read' | 'aws-exec-read' | 'bucket-owner-read' | 'bucket-owner-full-control',
    #     Body=b'bytes' | file,
    #     CacheControl='string',
    #     ContentDisposition='string',
    #     ContentEncoding='string',
    #     ContentLanguage='string',
    #     ContentLength=123,
    #     ContentMD5='string',
    #     ContentType='string',
    #     ChecksumAlgorithm='CRC32' | 'CRC32C' | 'SHA1' | 'SHA256',
    #     ChecksumCRC32='string',
    #     ChecksumCRC32C='string',
    #     ChecksumSHA1='string',
    #     ChecksumSHA256='string',
    #     Expires=datetime(2015, 1, 1),
    #     GrantFullControl='string',
    #     GrantRead='string',
    #     GrantReadACP='string',
    #     GrantWriteACP='string',
    #     Metadata={
    #         'string': 'string'
    #     },
    #     ServerSideEncryption='AES256' | 'aws:kms',
    #     StorageClass='STANDARD' | 'REDUCED_REDUNDANCY' | 'STANDARD_IA' | 'ONEZONE_IA' | 'INTELLIGENT_TIERING' | 'GLACIER' | 'DEEP_ARCHIVE' | 'OUTPOSTS' | 'GLACIER_IR',
    #     WebsiteRedirectLocation='string',
    #     SSECustomerAlgorithm='string',
    #     SSECustomerKey='string',
    #     SSEKMSKeyId='string',
    #     SSEKMSEncryptionContext='string',
    #     BucketKeyEnabled=True | False,
    #     RequestPayer='requester',
    #     Tagging='string',
    #     ObjectLockMode='GOVERNANCE' | 'COMPLIANCE',
    #     ObjectLockRetainUntilDate=datetime(2015, 1, 1),
    #     ObjectLockLegalHoldStatus='ON' | 'OFF',
    #     ExpectedBucketOwner='string'
    # )
    upload_object_response = None
    try:
        object_body = "auto test"
        bucket_name = DataGenerator.generate_bucket_name(context)
        context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
        # parse arguments
        kwargs = eval(parameters)
        for key, value in kwargs.items():
            if key == "Expires":
                expire_time = eval(value)
                kwargs[key] = str(expire_time)
        # put object
        if object_key:
            if object_key == "":
                upload_object_response = context.s3_resource.Object(bucket_name, key=object_key)
            else:
                if "Body" in kwargs and kwargs["Body"] == "":
                    pass
                else:
                    file_path = os.path.join(os.environ["test_templates_directory"], object_key)
                    if os.path.exists(file_path):
                        object_body = open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb')
                        kwargs["Body"] = object_body
                    else:
                        kwargs["Body"] = object_body
                # upload object
                upload_object_response = context.s3_resource.Object(bucket_name, object_key).put(**kwargs)
        log_message = "Put Object and get the response: " + str(
            str(upload_object_response))
        assert upload_object_response['ResponseMetadata']['HTTPStatusCode'] == 200
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ce:
        error_code = ce.response['Error']['Code']
        error_message = ce.response['Error']['Message']
        http_status_code = int(ce.response['ResponseMetadata']['HTTPStatusCode'])
        if 400 <= http_status_code < 500:
            log_message = f"Invoke request failure caused by client side error , error code '" + error_code + "' \n And error message '" + error_message + "'"
            context.logger.exception(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        elif http_status_code >= 500:
            log_message = f"Invoke request failure caused by server side error, error code '" + error_code + "' \n And error message '" + error_message + "' " + str(
                ce)
            context.logger.exception(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        else:
            log_message = f"Invoke request failure caused by other issue, error code '" + error_code + "' \n And error message '" + error_message + "'"
            context.logger.exception(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_message)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I update object ACL to bucket with key {object_key} and parameters {parameters}")
def step_impl(context, object_key, parameters):
    try:
        object_body = "auto test"
        bucket_name = DataGenerator.generate_bucket_name(context)
        context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
        # parse arguments
        kwargs = eval(parameters)
        for key, value in kwargs.items():
            if key == "Expires":
                expire_time = eval(value)
                kwargs[key] = str(expire_time)
        # put object
        if object_key:
            if object_key == "":
                update_object_acl_response = context.s3_resource.Object(bucket_name, key=object_key)
            else:
                if "Body" in kwargs and kwargs["Body"] == "":
                    pass
                else:
                    file_path = os.path.join(os.environ["test_templates_directory"], object_key)
                    if os.path.exists(file_path):
                        object_body = open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb')
                        kwargs["Body"] = object_body
                    if "Grant" in str(kwargs):
                        source_string = str(parameters)
                        variables_to_replace = re.findall("{{.*?}}", source_string)
                        context.logger.debug(variables_to_replace)
                        for variable in variables_to_replace:
                            key = str(variable).replace("{{", "").replace("}}", "")
                            canonical_id = eval(os.environ[key])['canonical_id']
                        kwargs = eval(source_string.replace(variables_to_replace[0], canonical_id))
                # upload object
                upload_object_response = context.s3_resource.Object(bucket_name, object_key).put(Body=object_body)
                if upload_object_response:
                    if "Body" in str(kwargs):
                        del kwargs["Body"]
                    update_object_acl_response = context.s3_resource.ObjectAcl(bucket_name, object_key).put(**kwargs)
                    if "ACL" in kwargs:
                        # get bucket acl grants
                        grants_response = context.s3_resource.ObjectAcl(bucket_name, object_key).grants
                        context.logger.debug("get object acl's grants_response: " + str(grants_response))
                        # get bucket acl owner
                        owner_response = context.s3_resource.ObjectAcl(bucket_name, object_key).owner
                        context.logger.debug("get object acl's owner_response: " + str(owner_response))
                    if "Grant" in str(kwargs):
                        # get bucket acl grants
                        grants_response = context.s3_resource.ObjectAcl(bucket_name, object_key).grants
                        context.logger.debug("Grant Control: object acl's grants_response: " + str(grants_response))
                        # get bucket acl owner
                        owner_response = context.s3_resource.ObjectAcl(bucket_name, object_key).owner
                        context.logger.debug("Grant Control: object acl's owner_response: " + str(owner_response))
        log_message = "Put Object and get the response: " + str(
            str(update_object_acl_response))
        assert update_object_acl_response['ResponseMetadata']['HTTPStatusCode'] == 200
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        assert update_object_acl_response['ResponseMetadata']['HTTPStatusCode'] == 200
    except ClientError as ce:
        error_code = ce.response['Error']['Code']
        error_message = ce.response['Error']['Message']
        http_status_code = int(ce.response['ResponseMetadata']['HTTPStatusCode'])
        if 400 <= http_status_code < 500:
            log_message = f"Invoke request failure caused by client side error , error code '" + error_code + "' \n And error message '" + error_message + "'"
            context.logger.exception(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        elif http_status_code >= 500:
            log_message = f"Invoke request failure caused by server side error, error code '" + error_code + "' \n And error message '" + error_message + "' " + str(
                ce)
            context.logger.exception(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        else:
            log_message = f"Invoke request failure caused by other issue, error code '" + error_code + "' \n And error message '" + error_message + "'"
            context.logger.exception(log_message)
            TestLog.attach_allure_log(
                log_message=log_message,
                log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_message)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I upload Path object to bucket with key {object_key} and parameters {parameters}")
def step_impl(context, object_key, parameters):
    """
    Upload object with parameters
    :param context:
    :param object_key:
    :param parameters:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context)
    context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
    # parse arguments
    kwargs = eval(parameters)
    for key, value in kwargs.items():
        if key == "Expires":
            expire_time = eval(value)
            context.logger.debug("expire_time: " + str(expire_time))
            kwargs[key] = expire_time
    # put object
    upload_object_response = context.s3_resource.Object(bucket_name, object_key).put(**kwargs)
    log_message = "Put Object and get the response: " + str(
        str(upload_object_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    assert upload_object_response['ResponseMetadata']['HTTPStatusCode'] == 200


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I upload Path object to bucket with deep path and parameters {parameters}")
def step_impl(context, parameters):
    """
    Upload folder object with parameters
    :param context:
    :param parameters:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context)
    context.logger.debug("======> start to upload objects to the bucket " + bucket_name)
    context.logger.debug("======> start to create the bucket " + bucket_name)
    # context.s3_resource.Bucket(bucket_name).create()
    context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
    context.logger.debug("======> end to create the bucket " + bucket_name)
    # parse arguments
    kwargs = eval(parameters)
    for key, value in kwargs.items():
        if key == "Expires":
            expire_time = eval(value)
            context.logger.debug("expire_time: " + str(expire_time))
            kwargs[key] = expire_time
    # generate object key/path/folder
    object_key = DataGenerator.generate_object_name(path_obj=True)
    # put object
    upload_object_response = context.s3_resource.Object(bucket_name, object_key).put(**kwargs)
    log_message = "Put Object and get the response: " + str(
        str(upload_object_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    assert upload_object_response['ResponseMetadata']['HTTPStatusCode'] == 200


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I upload object with key {object_key} to bucket {bucket_name}")
def step_impl(context, object_key, bucket_name):
    """
    Upload object with specified key
    :param context:
    :param object_key:
    :param bucket_name:
    :return:
    """
    if bucket_name:
        os.environ["bucket_name"] = bucket_name
        context.bucket_name = bucket_name
    else:
        bucket_name = DataGenerator.generate_bucket_name(context)
    context.logger.debug("======> start to upload objects to the bucket " + bucket_name)
    create_bucket_response = context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
    TestLog.attach_allure_log(
        log_message="Create Bucket Successfully and return the http status code: " + str(create_bucket_response),
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    upload_object_response = context.s3_resource.Object(bucket_name, object_key).put(
        Body=open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb'))
    context.logger.debug("upload object's response: " + str(upload_object_response))
    context.logger.debug("upload object's status: " + str(upload_object_response['ResponseMetadata']['HTTPStatusCode']))
    context.logger.debug("======> end to upload objects to the bucket " + bucket_name)
    log_message = "Put Object and get the response: " + str(
        str(upload_object_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    assert upload_object_response['ResponseMetadata']['HTTPStatusCode'] == 200


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I upload object with correct MD5 with key {object_key} and parameters {parameters}")
def step_impl(context, object_key, parameters):
    bucket_name = DataGenerator.generate_bucket_name(context)
    context.logger.debug("======> start to upload objects to the bucket " + bucket_name)
    context.logger.debug("======> start to create the bucket " + bucket_name)
    # context.s3_resource.Bucket(bucket_name).create()
    context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
    context.logger.debug("======> end to create the bucket " + bucket_name)
    # parse arguments
    kwargs = eval(parameters)
    context.logger.debug(kwargs)
    for key, val in kwargs.items():
        content_md5 = DataGenerator.calculate_content_md5(val)
    kwargs["ContentMD5"] = content_md5
    # put object
    upload_object_response = context.s3_resource.Object(bucket_name, object_key).put(**kwargs)
    log_message = "Put Object and get the response: " + str(
        str(upload_object_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)

    assert upload_object_response['ResponseMetadata']['HTTPStatusCode'] == 200


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I upload object with incorrect MD5 with key {object_key} and parameters {parameters}")
def step_impl(context, object_key, parameters):
    """
    Upload object with incorrect MD5
    :param context:
    :param object_key:
    :param parameters:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context)
    context.logger.debug("======> start to upload objects to the bucket " + bucket_name)
    context.logger.debug("======> start to create the bucket " + bucket_name)
    # context.s3_resource.Bucket(bucket_name).create()
    context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
    context.logger.debug("======> end to create the bucket " + bucket_name)
    # parse arguments
    kwargs = eval(parameters)
    for key, val in kwargs.items():
        content_md5 = DataGenerator.calculate_content_md5(val + "wrong content-md5")
    kwargs["ContentMD5"] = content_md5
    # put object
    try:
        context.s3_resource.Object(bucket_name, object_key).put(**kwargs)
    except ClientError as ex:
        context.error = str(ex)
        context.logger.debug("Expected error is found: " + str(ex))


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I upload object with key {object_key} to non-existing bucket")
def step_impl(context, object_key):
    """
    Upload object to non-existing bucket
    :param context:
    :param object_key:
    :return:
    """
    try:
        bucket_name = DataGenerator.generate_bucket_name(context)
        context.logger.debug("======> start to upload objects to the bucket " + bucket_name)
        file_path = os.path.join(os.environ["test_templates_directory"], object_key)
        if os.path.exists(file_path):
            response = context.s3_resource.Object(bucket_name, object_key).put(
                Body=open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb'))
        else:
            response = context.s3_resource.Object(bucket_name, object_key).put(
                Body="")
        context.logger.debug("======> end to upload objects to the bucket " + bucket_name)
        log_message = "Put Object and get the response: " + str(
            str(response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    except ClientError as ex:
        context.error = str(ex)
        context.logger.debug(str(ex))


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I validate error {expected_status_code} when upload object to non-existing bucket")
def step_impl(context, expected_status_code):
    """
    Validate error message
    :param context:
    :param expected_status_code:
    :return:
    """
    if context.error:
        if expected_status_code in context.error:
            TestLog.attach_allure_log(
                log_message=f"Put object to non-existing bucket and get the expected error: \n" + str(context.error),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        else:
            raise Exception(
                f"The expected error <" + expected_status_code + "> is not returned as expected in the response: \n" + str(
                    context.error))


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I upload object with key {object_key}")
def step_impl(context, object_key):
    """
    Create bucket and upload object to it
    :param context:
    :param object_key:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context)
    context.logger.debug("======> start to upload objects to the bucket " + bucket_name)
    # context.s3_resource.Bucket(bucket_name).create()
    context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
    upload_object_response = context.s3_resource.Object(bucket_name, object_key).put(
        Body=open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb'))
    context.logger.debug("======> end to upload objects to the bucket " + bucket_name)
    context.logger.debug("upload object's response: " + str(upload_object_response))
    context.logger.debug("upload object's status: " + str(upload_object_response['ResponseMetadata']['HTTPStatusCode']))
    context.logger.debug("======> end to upload objects to the bucket " + bucket_name)
    log_message = "Put Object and get the response: " + str(
        str(upload_object_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    assert upload_object_response['ResponseMetadata']['HTTPStatusCode'] == 200


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I create folder for the bucket {bucket_name}")
def step_impl(context, folder_name, bucket_name):
    context.logger.debug("======> start to crate folder for the bucket " + bucket_name)
    upload_folder_response = context.s3_resource.Object(bucket_name, folder_name).put(
        Key=folder_name)
    context.logger.debug("create folder's response: " + str(upload_folder_response))
    context.logger.debug("======> end to crate folder for the bucket " + bucket_name)
    log_message = "Create folder for the bucket and get the response: " + str(
        str(upload_folder_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    assert upload_folder_response['ResponseMetadata']['HTTPStatusCode'] == 200


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I create the folder {folder_name} for one bucket")
def step_impl(context, folder_name):
    if os.environ["bucket_name"]:
        bucket_name = os.environ["bucket_name"]
        context.logger.debug("bucket name: " + bucket_name)
    else:
        raise Exception("Bucket name is not found as expected!")
    context.logger.debug("======> start to create folder for the bucket " + bucket_name)
    upload_folder_response = context.s3_resource.Object(bucket_name, folder_name).put(
        Key=folder_name)
    context.logger.debug("create folder's response: " + str(upload_folder_response))
    context.logger.debug("======> end to create folder for the bucket " + bucket_name)
    log_message = "Create folder for the bucket and get the response: " + str(
        str(upload_folder_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
    assert upload_folder_response['ResponseMetadata']['HTTPStatusCode'] == 200


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete object from bucket")
def step_impl(context):
    bucket_name = context.bucket_name
    delete_object_responses = context.boto3_client.s3_delete_all_objects(bucket_name=bucket_name)
    for delete_object_response in delete_object_responses:
        context.logger.debug(
            "delete object's status: " + str(delete_object_response['ResponseMetadata']['HTTPStatusCode']))
        log_message = f"Delete objects and get the response: \n" + str(delete_object_response)
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        # assert delete_object_response['ResponseMetadata']['HTTPStatusCode'] == 204

    context.logger.debug("======> end to delete objects from the bucket" + bucket_name)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete object with key {object_key} from bucket")
def step_impl(context, object_key):
    bucket_name = context.bucket_name
    context.logger.debug("======> start to delete objects from the bucket " + bucket_name)
    context.boto3_client.s3_resource_delete_single_object(bucket_name=bucket_name, object_key=object_key)
    context.logger.debug("======> end to delete objects from the bucket " + bucket_name)
    log_message = "Delete objects '" + object_key + "' from the bucket '" + bucket_name + "'"
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXECUTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete object with versions from bucket")
def step_impl(context, object_key=None):
    bucket_name = context.bucket_name
    context.logger.debug("======> start to delete objects from the bucket")
    s3_bucket = context.s3_resource.Bucket(bucket_name)
    context.logger.debug(f'Deleting {bucket_name} ...')
    try:
        if object_key:
            context.boto3_client.s3_resource_delete_single_object(bucket_name=bucket_name, object_key=object_key)
            context.logger.debug(f"Permanently deleted all versions of object {object_key} in {bucket_name}.", )
        else:
            s3_bucket.object_versions.delete()
            context.logger.debug(f"Permanently deleted all versions of all objects in {bucket_name}.")
    except ClientError as ex:
        context.logger.debug(f"Couldn't delete all versions of {object_key} in {bucket_name}.")
        raise ex


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I verify expected error {expected_error} is found")
def step_impl(context, expected_error):
    actual_error = context.error
    if str(expected_error) in str(
            actual_error) or "An error occurred (PreconditionFailed) when calling the CopyObject operation: At least one of the preconditions you specified did not hold" in str(
        actual_error) or "An error occurred (InvalidRequest) when calling the PutObject operation" in str(actual_error):
        context.logger.debug("Expected error is found: " + str(expected_error))
    else:
        raise Exception(
            f"Expected error <" + expected_error + "> is not found in the response:\n <" + actual_error + ">")
