import os

from behave import *
from botocore.exceptions import ClientError

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog
from testbase.teamplates.bucket_acl_template import BucketAclTemplate


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When(
    "I Put Object Acl|AccessControlPolicy to tenant user:{owner_display_name},{owner_id},{grantee_display_name},{grantee_id},{grantee_type},{grantee_permission},{object_key}")
def step_impl(context, owner_display_name, owner_id, grantee_display_name, grantee_id, grantee_type,
              grantee_permission, object_key):
    """
    Put Object ACL to tenant user
    :param context:
    :param owner_display_name:
    :param owner_id:
    :param grantee_display_name:
    :param grantee_id:
    :param grantee_type:
    :param grantee_permission:
    :param object_key:
    :return:
    """
    # AccessControlPolicy = {
    #     'Grants': [
    #         {
    #             'Grantee': {
    #                 'DisplayName': 'string',
    #                 'EmailAddress': 'string',
    #                 'ID': 'string',
    #                 'Type': 'CanonicalUser' | 'AmazonCustomerByEmail' | 'Group',
    #                 'URI': 'string'
    #             },
    #             'Permission': 'FULL_CONTROL' | 'WRITE' | 'WRITE_ACP' | 'READ' | 'READ_ACP'
    #         },
    #     ],
    #     'Owner': {
    #         'DisplayName': 'string',
    #         'ID': 'string'
    #     }
    # }
    try:
        if context.bucket_name:
            bucket_name = context.bucket_name
        else:
            bucket_name = DataGenerator().id_generator_lowercase(size=15)
            os.environ["bucket_name"] = bucket_name
            context.bucket_name = bucket_name
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        # Add object to bucket
        put_object_response = context.s3_resource.Object(bucket_name, object_key).put(
            Body=open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb'))
        TestLog.attach_allure_log(
            log_message="Add object's response: " + str(put_object_response),
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        # Get value from global environment variables
        owner_display_name = eval(os.environ[owner_display_name])['canonical_id']
        owner_id = eval(os.environ[owner_id])['canonical_id']
        grantee_display_name = eval(os.environ[grantee_display_name])['canonical_id']
        grantee_id = eval(os.environ[grantee_id])['canonical_id']
        parameters = {"owner_display_name": owner_display_name, "owner_id": owner_id,
                      "grantee_display_name": grantee_display_name, "grantee_id": grantee_id,
                      "grantee_type": grantee_type, "grantee_permission": grantee_permission}

        # Put Object ACL
        object_acl = BucketAclTemplate.get_bucket_access_control_policy(**parameters)
        TestLog.attach_allure_log(
            log_message="Object ACL request payload: " + str(object_acl),
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        put_object_acl_response = context.s3_resource.ObjectAcl(bucket_name, object_key).put(
            AccessControlPolicy=object_acl
        )
        http_status_code = str(put_object_acl_response['ResponseMetadata']['HTTPStatusCode'])
        TestLog.attach_allure_log(
            log_message="Add Object ACL: " + str(object_acl) + " And get status code: " + str(http_status_code),
            log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
        context.actual_status_code_create_bucket = http_status_code

    except ClientError as ex:
        log_error = "Put Object ACL and get the exception: " + str(ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=str(log_error),
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When(
    "I Put Object ACL to All Users:{uri}, {owner_id},{grantee_id},{grantee_type},{grantee_permission},{object_key}")
def step_impl(context, uri, owner_id, grantee_id, grantee_type,
              grantee_permission, object_key):
    """
    Put Object to All users
    :param context:
    :param uri:
    :param owner_id:
    :param grantee_id:
    :param grantee_type:
    :param grantee_permission:
    :param object_key:
    :return:
    """
    try:
        if context.bucket_name:
            bucket_name = context.bucket_name
        else:
            bucket_name = DataGenerator().id_generator_lowercase(size=15)
            os.environ["bucket_name"] = bucket_name
            context.bucket_name = bucket_name
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        put_object_response = context.s3_resource.Object(bucket_name, object_key).put(
            Body=open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb'))
        if put_object_response:
            # get value from global environment variables
            owner_display_name = eval(os.environ[owner_id])['canonical_id']
            owner_id = eval(os.environ[owner_id])['canonical_id']
            grantee_display_name = eval(os.environ[grantee_id])['canonical_id']
            grantee_id = eval(os.environ[grantee_id])['canonical_id']
            parameters = {"uri": uri, "owner_display_name": owner_display_name, "owner_id": owner_id,
                          "grantee_display_name": grantee_display_name, "grantee_id": grantee_id,
                          "grantee_type": grantee_type, "grantee_permission": grantee_permission}

            object_acl = BucketAclTemplate.get_bucket_acl_for_all_users(**parameters)
            TestLog.attach_allure_log(
                log_message="Object ACL request payload: " + str(object_acl),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            response = context.s3_resource.ObjectAcl(bucket_name, object_key).put(
                AccessControlPolicy=object_acl
            )
            http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
            TestLog.attach_allure_log(
                log_message="Add object ACL: " + str(object_acl) + " And get status code: " + str(http_status_code),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            context.actual_status_code_create_bucket = http_status_code

    except Exception as ex:
        raise Exception("Put Object ACL: " + str(ex))


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When(
    "I Put Object ACL to Authenticated Users:{uri}, {owner_id},{grantee_id},{grantee_type},{grantee_permission},{object_key}")
def step_impl(context, uri, owner_id, grantee_id, grantee_type,
              grantee_permission, object_key):
    """
    Put object ACL to Authenticated Users
    :param context:
    :param uri:
    :param owner_id:
    :param grantee_id:
    :param grantee_type:
    :param grantee_permission:
    :param object_key:
    :return:
    """
    try:
        if context.bucket_name:
            bucket_name = context.bucket_name
        else:
            bucket_name = DataGenerator.generate_bucket_name(context)
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        if context.boto3_client.s3_waiters_check_bucket_exists(bucket_name=bucket_name):
            put_object_response = context.s3_resource.Object(bucket_name, object_key).put(
                Body=open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb'))
            context.logger.debug("======> put object response: " + str(put_object_response) + " <======")
        # get value from global environment variables
        owner_display_name = eval(os.environ[owner_id])['canonical_id']
        owner_id = eval(os.environ[owner_id])['canonical_id']
        grantee_display_name = eval(os.environ[grantee_id])['canonical_id']
        grantee_id = eval(os.environ[grantee_id])['canonical_id']
        parameters = {"uri": uri, "owner_display_name": owner_display_name, "owner_id": owner_id,
                      "grantee_display_name": grantee_display_name, "grantee_id": grantee_id,
                      "grantee_type": grantee_type, "grantee_permission": grantee_permission}
        if context.boto3_client.s3_waiters_check_object_exists(bucket_name=bucket_name, key_name=object_key):
            object_acl = BucketAclTemplate.get_bucket_acl_for_all_users(**parameters)
            TestLog.attach_allure_log(
                log_message="Object ACL request payload: " + str(object_acl),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            response = context.s3_resource.ObjectAcl(bucket_name, object_key).put(
                AccessControlPolicy=object_acl
            )
            http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
            TestLog.attach_allure_log(
                log_message="Add Object ACL: " + str(object_acl) + " And get status code: " + str(http_status_code),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            context.actual_status_code_create_bucket = http_status_code
    except ClientError as ex:
        log_error = "Put Object ACL and get the exception: " + str(ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=str(log_error),
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then(
    "I Put Object ACL to tenant user: {object_acl},{object_key}")
def step_impl(context, object_acl, object_key):
    """
    Put Object ACL to tenant user
    :param context:
    :param object_acl:
    :param object_key:
    :return:
    """
    try:
        if context.bucket_name:
            bucket_name = context.bucket_name
        else:
            bucket_name = DataGenerator.generate_bucket_name(context)
        context.boto3_client.s3_create_bucket(bucket_name=bucket_name)
        put_object_response = context.s3_resource.Object(bucket_name, object_key).put(
            Body=open(os.path.join(os.environ["test_templates_directory"], object_key), 'rb'))
        if put_object_response:
            TestLog.attach_allure_log(
                log_message="Object ACL request payload: " + str(object_acl),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            response = context.s3_resource.ObjectAcl(bucket_name, object_key).put(
                ACL=object_acl
            )
            http_status_code = str(response['ResponseMetadata']['HTTPStatusCode'])
            TestLog.attach_allure_log(
                log_message="Add Object ACL: " + str(object_acl) + "and get status code: " + str(http_status_code),
                log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            context.actual_status_code_create_bucket = http_status_code

    except ClientError as ex:
        log_error = "Put Object ACL and get the exception: " + str(ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=str(log_error),
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I get Object ACL:{object_key}")
def step_impl(context, object_key):
    """
    Get Object ACL with key
    :param context:
    :param object_key:
    :return:
    """
    try:
        bucket_name = context.bucket_name
        # wait for object exists
        if context.boto3_client.s3_waiters_check_object_exists(bucket_name=bucket_name, key_name=object_key):
            # get bucket acl grants
            grants_response = context.s3_resource.ObjectAcl(context.bucket_name, object_key).grants
            if grants_response:
                TestLog.attach_allure_log(
                    log_message="get object ACL and get grants: " + str(grants_response),
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            else:
                raise Exception("Fail to grants of Bucket ACL for bucket '" + bucket_name + "'")
            # get bucket acl owner
            owner_response = context.s3_resource.ObjectAcl(context.bucket_name, object_key).owner
            if grants_response:
                TestLog.attach_allure_log(
                    log_message="get object ACL and get owner: " + str(owner_response),
                    log_type=TestLog.LOG_TYPE_TEST_EXECUTION)
            else:
                raise Exception("Fail to owner of Bucket ACL for bucket '" + bucket_name + "'")
    except ClientError as ex:
        log_error = "Get Object ACL and get the exception: " + str(ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=str(log_error),
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)
