import urllib

from behave import *

from common_libraries.data_generator import DataGenerator
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@When("I upload object tagging with correct MD5 with key {object_key} and parameters {parameters}")
def step_impl(context, object_key, parameters):
    """
    Upload object with tag
    :param context:
    :param object_key:
    :param parameters:
    :return:
    """
    bucket_name = DataGenerator.generate_bucket_name(context)
    context.boto3_client.s3_resource_create_bucket(bucket_name=bucket_name)
    # parse arguments
    tags = eval(parameters)
    tagging = urllib.parse.urlencode(tags)
    put_obj_tagging_response = context.s3_client.put_object(
        Bucket=bucket_name,
        Key=object_key,
        Tagging=tagging
    )

    http_status_code = put_obj_tagging_response["ResponseMetadata"]["HTTPStatusCode"]
    assert int(http_status_code) == 200
    log_message = "Put Object Tagging: " + str(
        str(put_obj_tagging_response))
    context.logger.debug(log_message)
    TestLog.attach_allure_log(
        log_message=log_message,
        log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I get object tagging with key {object_key}")
def step_impl(context, object_key):
    """
    Get object tag
    :param context:
    :param object_key:
    :return:
    """
    bucket_name = context.bucket_name
    get_obj_tagging_response = context.s3_client.get_object_tagging(
        Bucket=bucket_name,
        Key=object_key
    )
    http_status_code = get_obj_tagging_response["ResponseMetadata"]["HTTPStatusCode"]
    assert int(http_status_code) == 200
    if get_obj_tagging_response:
        log_message = "Get Object Tagging: " + str(
            str(get_obj_tagging_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I delete object tagging with key {object_key}")
def step_impl(context, object_key):
    """
    Delete object tag
    :param context:
    :param object_key:
    :return:
    """
    bucket_name = context.bucket_name
    delete_obj_tagging_response = context.s3_client.delete_object_tagging(
        Bucket=bucket_name,
        Key=object_key
    )
    http_status_code = delete_obj_tagging_response["ResponseMetadata"]["HTTPStatusCode"]
    assert int(http_status_code) == 204
    if delete_obj_tagging_response:
        log_message = "Delete Object Tagging: " + str(
            str(delete_obj_tagging_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
