from behave import *
from botocore.exceptions import ClientError
from common_libraries.decorator import Decorator
from common_libraries.logger import TestLog


def _add_header(request):
    request.headers.add_header('Content-Type', "application/json")


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should put public access block successfully with configuration: {public_access_block_configuration}")
def step_impl(context, public_access_block_configuration):
    """
    Put public access block
    :param context:
    :param public_access_block_configuration:
    :return:
    """
    # Request Syntax
    # response = client.put_public_access_block(
    #     Bucket='string',
    #     ContentMD5='string',
    #     ChecksumAlgorithm='CRC32' | 'CRC32C' | 'SHA1' | 'SHA256',
    #     PublicAccessBlockConfiguration={
    #         'BlockPublicAcls': True | False,
    #         'IgnorePublicAcls': True | False,
    #         'BlockPublicPolicy': True | False,
    #         'RestrictPublicBuckets': True | False
    #     },
    #     ExpectedBucketOwner='string'
    # )
    try:
        bucket_name = context.bucket_name
        event_system = context.s3_client.meta.events
        event_system.register_first('before-sign.*.*', lambda request, **kwargs: _add_header(request))
        # put_public_access_block
        put_public_access_block_response = context.s3_client.put_public_access_block(Bucket=bucket_name,
                                                                                     PublicAccessBlockConfiguration=eval(
                                                                                         public_access_block_configuration))

        log_message = "put_public_access_block(): " + str(
            str(put_public_access_block_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
    except ClientError as ex:
        log_error = f"Fail to put_public_access_block('" + public_access_block_configuration + "')' failed \n and get unexpected exception: " + str(
            ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should be able to get public access block successfully")
def step_impl(context):
    """
    Get public access block
    :param context:
    :return:
    """
    # Request Syntax
    # response = client.get_public_access_block(
    #     Bucket='string',
    #     ExpectedBucketOwner='string'
    # )
    kwargs = {}
    try:
        bucket_name = context.bucket_name
        # parse arguments
        kwargs = {"Bucket": bucket_name}
        # get_public_access_block
        get_public_access_block_response = context.s3_client.get_public_access_block(**kwargs)
        log_message = "get_public_access_block(): " + str(
            str(get_public_access_block_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
    except ClientError as ex:
        log_error = f"Fail to get_public_access_block('" + str(
            kwargs) + "') \n and get the unexpected exception " + str(
            ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)


@Decorator.log(my_logger=TestLog())
@Decorator.exception_handler()
@Then("I should be able to delete public access block successfully")
def step_impl(context):
    """
    Delete public access block
    :param context:
    :return:
    """
    # Request Syntax
    # response = client.delete_public_access_block(
    #     Bucket='string',
    #     ExpectedBucketOwner='string'
    # )
    kwargs = {}
    try:
        bucket_name = context.bucket_name
        # parse arguments
        kwargs = {"Bucket": bucket_name}
        # delete_public_access_block
        delete_public_access_block_response = context.s3_client.delete_public_access_block(**kwargs)
        log_message = "delete_public_access_block(): " + str(
            str(delete_public_access_block_response))
        context.logger.debug(log_message)
        TestLog.attach_allure_log(
            log_message=log_message,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
    except ClientError as ex:
        log_error = f"Fail to delete_public_access_block('" + str(
            kwargs) + "') \n and get the unexpected exception " + str(
            ex)
        context.logger.error(log_error)
        TestLog.attach_allure_log(
            log_message=log_error,
            log_type=TestLog.LOG_TYPE_TEST_EXCEPTION)
        raise Exception(log_error)
