#!/bin/sh
if [ -z "$1" ]; then
  echo "No parameter <test configuration> passed."
else
  echo "Parameter passed <test configuration> = $1"
fi

echo "Go to working directory"
cd /certs/ose-osis-cert221/ose-221 || exit
export ose_osis=.
# Log, delete existing logs
file=$ose_osis/logs/ose.log
if [ -f "$file" ]; then
  find $ose_osis/logs/ose.log* -mtime +2 -type f -name "*.log" -delete
fi

# Agent, get log directory and test case name
. /certs/common/launcher.sh
LOGDIR=$(getLogDir)
TCNAME=$(getTestCaseName)

# Log passed test
ResultPass() {
  result=$LOGDIR/.result
  cat >"$result" <<-"EOF"
passed
EOF
}

LogPass() {
  run=$LOGDIR/run.log
  cat >"$run" <<-"EOF"
passed
EOF
}

# Log failed test
ResultFail() {
  result=$LOGDIR/.result
  cat >"$result" <<-"EOF"
failed
EOF
}

LogFail() {
  run=$LOGDIR/run.log
  cat >"$run" <<-"EOF"
failed
EOF
}

# Return code for if no launcher found
# shellcheck disable=SC2181
if [ $? -eq 0 ]; then
  echo "The launcher script ran ok"
else
  echo "The launcher script failed" >&2
  ResultFail
  LogFail
  exit 1
fi

input="/certcontent/ose.log"
runList="/runlist.json"

Check() {
  while IFS= read -r line; do
    if [ "$line" = "AllTestSuite:failed" ]; then
      ResultFail
      LogFail
    fi
  done <"$input"
}

# Delete the previous session failed AllTestSuite test
# shellcheck disable=SC2039
if [ "$TCNAME" == "AllTestSuite" ] && [ -f "$input" ]; then
  rm -f $input
fi

if [ -f "$input" ]; then
  Check
  exit 1
fi

if [ ! -f "$runList" ]; then
  ResultFail
  LogFail
  exit 1
fi

# Remove the previous automation test log and test report
viva_agent_directory=$LOGDIR
my_test_log_directory=$viva_agent_directory/test_log
my_test_report_directory=$viva_agent_directory/test_report

echo "Test log directory: my_test_log_directory"
echo "Test report directory: my_test_report_directory"
# test log directory
if [ -d "$my_test_log_directory" ]; then
  echo "Delete new directory: $my_test_log_directory"
  rm -rf "$my_test_log_directory"
else
  echo "create new directory: $my_test_log_directory"
  mkdir -p "$my_test_log_directory"
fi

# test report directory
if [ -d "$my_test_report_directory" ]; then
  echo "Delete new directory: $my_test_report_directory"
  rm -rf "$my_test_report_directory"
else
  echo "create new directory: $my_test_report_directory"
  mkdir -p "$my_test_report_directory"
fi

echo "Automation Test Log: $my_test_log_directory"
echo "Automation Test Report: $my_test_report_directory"

working_dir=/certs/ose-osis-cert221/ose-221
if [ -d $working_dir ]; then
  echo "Install Python tools"
  tdnf install -y python3-pip python3-setuptools
  echo "Install Python packages"
  pip3 install -r requirements.txt
  echo "Install Nodejs"
  tdnf -y install nodejs
  npm --version
  echo "Install Allure command line"
  npm install -g allure-commandline --save-dev
  allure --version
  echo "Install jq"
  tdnf install -y jq
  jq --version
  # Load Test parameters from runlist.json
  vcd_endpoint=$(jq '.params.testParams[] .params_display.VCDHost.value' /runlist.json)
  echo "$vcd_endpoint"
  ose_endpoint=$(jq '.params.testParams[] .params_display.OSEEndpoint.value' /runlist.json)
  echo "$ose_endpoint"
  yml_file=$(jq '.params.testParams[] .params_display.OSEPlatformSelection.value' /runlist.json)
  yml_file=$yml_file".yml"
  echo "$yml_file"
  echo "======> Start to Run automation test suites ======>"
  # Test Parameters
  vcd_host=$vcd_endpoint
  ose_endpoint=$ose_endpoint
  environment_file=$yml_file
  test_configuration=$1
  if [ -z "$test_configuration" ]; then
    python3 ose_test_runner.py --env_file="$environment_file" --test_report_directory="$my_test_report_directory" --test_log_directory="$my_test_log_directory" --project_name=common_test_suite --vcd_endpoint="$vcd_host" --ose_endpoint="$ose_endpoint" --bucket_prefix="ose-api-test-"
    ResultPass
    LogPass
  else
    python3 ose_test_runner.py --env_file="$environment_file" --test_report_directory="$my_test_report_directory" --test_log_directory="$my_test_log_directory" --project_name=common_test_suite --vcd_endpoint="$vcd_host" --ose_endpoint="$ose_endpoint" --bucket_prefix="ose-api-test-" --test_configuration="$test_configuration"
    ResultPass
    LogPass
  fi

  echo "======> End to Run automation test suites ======>"
  exit 0
else
  echo "Working directory is not found."
fi
